#' @title Aggregate damage
#' 
#' @description
#' `aggregate_damage()` reads combined damage functions and aggregates damage 
#' accordingly to the hazard parameters in `hazard_agg`.
#'
#' @param path list, describing where to find useful paths.
#' @param damage_origin character indicating which type of damage should be
#' aggregated. The damage functions to aggregate should be found in one of 
#' `list.files(path[["output"]][["combine"]])` after having called 
#' `combine_damaging()`. Default to "total".
#' @param hazard_agg list of data.frames which contains the aggregated hazard
#' parameters. Default to `floodam.agri::HAZARD_AGGREGATE` used to build the
#' french national damage functions.
#' @param retrieve logical indicating whether the result should be returned. 
#' Default to TRUE.
#' 
#' @return data.frame of damage aggregated.
#'
#' @export
#'
#' @encoding UTF-8

aggregate_damage = function(
    path,
    damage_origin = "total",
    hazard_agg = floodam.agri::HAZARD_AGGREGATE,
    retrieve = TRUE
) {
    detail = utils::read.csv2(
        file.path(path[["output"]][["combine"]], paste0(damage_origin, ".csv")),
        check.names = FALSE
    )

    result = calculate_aggregated_damage(detail, hazard_agg)

    utils::write.csv2(
        result,
        file.path(
            path[["output"]][["combine"]],
            paste0(damage_origin, ".aggregate.csv")
        ),
        row.names = FALSE
    )
    if (retrieve) return(result)
}

calculate_aggregated_damage = function(detail, hazard_agg) {
    hazard_agg = lapply(hazard_agg, function(x) {x[order(x[["max"]]), ]})
    b = lapply(hazard_agg, function(x) {c(-1, x[["max"]])})
    l = lapply(hazard_agg, rownames)

    f = as.data.frame(mapply(cut, detail[names(hazard_agg)], b, l))

    result = stats::aggregate(
        detail[-c(1:4)],
        by = cbind(height = detail[["height"]], f),
        mean
    )
    return(result)
}
