#' @title Integrate weeks in 3D array
#'
#' @description 
#' `array_week()` transforms a list of 3D-arrays on a list of 4D-arrays
#' by taking into account information on week.
#' Called in `make_curve()`.
#' 
#' @param l list of 3D-arrays (height, duration, current) of damaging function 
#' for a list of phenomena.
#' @param calendar data.frame, calendar of `damaging.complement`.
#' @param week vector of values of the week where to calculate damaging.
#' @param type vector of type of phenomena used to group damaging function.
#' @param rule list of rules to be passed to each `type`.
#'
#' @return list of 4D-arrays (height, duration, current, week), elements of the
#' list are given by the type
#'
#' @export
#'
#' @encoding UTF-8

array_week = function(
    l,
    calendar,
    week,
    type = NULL,
    rule = list()
) {
    # function of how to combine multiple damaging functions
    combine = function(x, rule) {
        result = array(
            unlist(x),
            dim = c(dim(x[[1]]), length(x)),
            dimnames = c(dimnames(x[[1]]), list(names(x)))
        )
        margin = seq(length(dim(result)) - 1)
        if (rule == "max") return(apply(result, margin, max))
        if (rule == "sum") return(apply(result, margin, sum))
        if (rule == "geometric") return(apply(
            result,
            margin,
            function(x) {1 - prod(1 -x)}
        ))
        stop("Bad rule for combination")
    }

    local_array_week = function(name, l, week, rule) {
        result = lapply(name, function(x) {combine(l[x], rule)})
        array(
            unlist(result),
            dim = c(dim(result[[1]]), length(week)),
            dimnames = c(dimnames(result[[1]]), list(week = week))
        )
    }

    if (is.null(type)) {
        name = mapply(find_curve, week, list(calendar))
        return(local_array_week(name, l, week, rule))
    }

    name = lapply(
        unique(type), 
        function(x) {
            mapply(
                find_curve,
                week,
                list(calendar[type == x,]),
                SIMPLIFY = FALSE
            )
        }
    )
    names(name) = unique(type)
    rule = rule[unique(type)]
    result = mapply(
        local_array_week,
        name,
        list(l),
        list(week),
        rule,
        SIMPLIFY = FALSE
    )
    names(result) = unique(type)
    return(result)
}