#' @title Create damage curve
#' 
#' @description
#' `calculate_damaging()` creates a damage curve for one or more crops.
#'
#' @param path list describing where to find useful paths. Result of
#' `init_path()`.
#' @param culture character indicating which crop to consider, if NULL then all 
#' crops from the extent are considered, default to NULL.
#' @param level list giving levels for hazard parameters (current, duration, 
#' height, week), default to floodam.agri::LEVEL.
#' @param combine list of combine methods for each type of phenomena used to
#' group damaging function. Default to floodam.agri::COMBINE
#' @param current data.frame, correspondance between velocity qualitative 
#' classes and their numeric value. Default to floodam.agri::CURRENT.
#' @param hazard character, hazard parameters, default to floodam.agri::HAZARD
#' @param backup character indicating which calculations should be saved. Set to
#' NULL if no calculations should be saved. Default to `c("sumup", "full"). 
#' @param verbose logical, should the function send some messages while running.
#' @param file.log character, name of journal file. DEFAULT to "damage.log".
#' @param update.hazard logical, should the hazard be updated, default to FALSE.
#' @param retrieve logical, should result be returned. Default to FALSE.
#' @param simple logical, should result be returned as a data.frame or a list.
#' 
#' @return 
#' If `backup` is "sumup", `calculate_damaging()` saves two csv files to the 
#' following directory `file.path(path[["output"]][["damage"]], "sumup")` with
#' the same number of rows for each crop in `culture`:
#'  * `damage_%s.csv` which gives the damage function for total damage only
#' where %s indicates the name of the `culture`.
#'  * `damage-disaggregated_%s.csv` which gives the damage function for total
#' damage and damage to other components (yield, replanting, soil and material). 
#' 
#' If `backup` is `full`, a data.frame giving both proportions of damage to a 
#' component and total damage for each component is saved in 
#' `file.path(path[["output"]][["damage"]], "full")`.
#' 
#' If `retrieve` is TRUE, `calculate_damaging()` will return the damage function
#' for total damage and damage to other components (yield, replanting, soil and 
#' material).
#' 
#' If there is no `hazard.csv` file in `path[["output"]][["damage"]]`, a 
#' data.frame with the same amount of rows as the damage data.frames indicating 
#' the hazard characteristics is saved. To update this file, `update.hazard`.
#' should be set to TRUE.
#' 
#' @export
#'
#' @encoding UTF-8

calculate_damaging = function(
    path,
    culture = NULL,
    level = floodam.agri::LEVEL,
    combine = floodam.agri::COMBINE,
    current = floodam.agri::CURRENT,
    hazard = floodam.agri::HAZARD,
    backup = c("sumup", "full"),
    verbose = TRUE,
    file.log = "damage.log",
    update.hazard = FALSE,
    retrieve = FALSE,
    simple = FALSE
) {
    begin = Sys.time()

    if (is.null(culture)) culture = dir(path[["output"]][["culture"]])
    if (length(culture) > 1) {
        msg = sprintf("%s Update of culture damaging functions:", Sys.Date())
        if (isTRUE(verbose)) cat("Update of culture damaging functions:\n")
        write(msg, file.path(path[["output"]][["damage"]], file.log))
        damaging = stats::setNames(
            lapply(
                culture,
                calculate_damaging,
                path = path,
                level = level,
                combine = combine,
                current = current,
                backup = backup,
                verbose = verbose,
                file.log = file.log,
                update.hazard = update.hazard,
                simple = TRUE
            ),
            culture
        )
        elapsed = Sys.time() - begin
        msg = sprintf("... processed in %1.2f %s", elapsed, units(elapsed))

        if (isTRUE(verbose)) cat(msg, "\n")

        write(
            msg,
            file.path(path[["output"]][["damage"]], file.log),
            append = TRUE
        )

        if (isTRUE(retrieve)) {
            return(invisible(damaging))
        }
        return(NULL)
    }

    if (isTRUE(verbose)) {
        cat(sprintf("\tProcessing %s...", culture))
    }

    # 1: Recuperate all parameters
    parameter = load_parameter(path[["output"]][["culture"]], culture)

    # 2: Interpolate damaging curve to level given in floodam.agri::LEVEL
    calendar = fit_calendar(parameter[["calendar"]])
    result = make_curve(parameter$damaging.core,
        parameter$damaging.complement, calendar, level, combine, current)

    # 3: Calculating damage
    # 3.1: Loss of plant and yield
    result$plant.raw = inv(result, "damaging.plant")
    result$yield.raw = inv(result, "damaging.yield")
    result$yield.plant = result$plant.raw * result$yield.raw

    result = cbind(result,
        apply_test(parameter$rule, result, calendar))

    # 3.2: Actions
    result$plant.rule.inv = inv(result, "plant.rule")
    result$yield.rule.inv = inv(result, "yield.rule")
    result$yield.rule.price = ## Ajout ALA
        result$yield.rule.inv + 
        result$yield.rule * 
        result$damaging.price

    result$ratio.week.yieldrule =
        ratio_week_period(result$week, calendar) * 
        result$yield.rule.inv

    result = cbind(
        result, 
        apply_test(parameter$action, result, calendar, default = 0)
    )

    # 3.3: Damage
    result$total = result$damage.culture =
        apply(result[unique(as.character(parameter$action$object))], 1, sum)
    if ("damage.soil" %in% colnames(result))
        result$total = result$total + result$damage.soil
    if ("damage.material" %in% colnames(result))
        result$total = result$total + result$damage.material

    action = unique(as.character(parameter$action$object))
    differed = grep(".d$", action, value = TRUE)
    replanting = grep("replanting$", action, value = TRUE)
    annual = action[!(action %in% c(differed, replanting))]

    disaggregated = data.frame(
        total = result[["total"]],
        yield.annual = if (
            length(annual) > 0
        ) apply(result[annual], 1, sum) else 0,
        yield.differed = if (
            length(differed) > 0
        ) apply(result[differed], 1, sum) else 0,
        replanting = if (
            length(replanting) > 0
        ) apply(result[replanting], 1, sum) else 0,
        soil = if (
            "damage.soil" %in% colnames(result)
        ) result$damage.soil else 0,
        material = if (
            "damage.material" %in% colnames(result)
        ) result$damage.material else 0
    )

    # 4: Saving calculations
    if ("full" %in% backup) {
        full = file.path(path[["output"]][["damage"]], "full")
        dir.create(full, FALSE, TRUE)
        utils::write.csv2(
            result[!(colnames(result) %in% hazard)],
            file.path(full, sprintf("damage_%s.csv", culture)),
            row.names = FALSE
        )
    }

    if ("sumup" %in% backup) {
        sumup = file.path(path[["output"]][["damage"]], "sumup")
        dir.create(sumup, FALSE, TRUE)

        utils::write.csv2(
            result[c("total")],
            file.path(sumup, sprintf("damage_%s.csv", culture)),
            row.names = FALSE
        )
        utils::write.csv2(
            disaggregated,
            file.path(sumup, sprintf("damage-disaggregated_%s.csv", culture)),
            row.names = FALSE
        )
    }
    if (update.hazard | ! "hazard.csv" %in% dir(path[["output"]][["damage"]])) {
        utils::write.csv2(
            result[hazard],
            file.path(path[["output"]][["damage"]], "hazard.csv"),
            row.names = FALSE
        )
    }

    elapsed = Sys.time() - begin
    if (isTRUE(verbose)) {
        cat(sprintf(" Finished in %1.2f %s\n", elapsed, units(elapsed)))
    }

    msg = sprintf(
        "\t%s processed in %1.2f %s",
        culture,
        elapsed, 
        units(elapsed)
    )
    msg = c(
        msg, 
        sprintf(
            "\t\t-Annual damage contain: %s",
            paste(annual, collapse = ", ")
        )
    )
    msg = c(
        msg,
        sprintf(
            "\t\t-Differed damage contain: %s\n",
            paste(differed, collapse = ", ")
        )
    )
    if (file.log %in% dir(path[["output"]][["damage"]])) {
        write(
            msg,
            file.path(path[["output"]][["damage"]], file.log),
            append = TRUE
        )
    } else {
        msg = c(
            sprintf(
                "%s Update of culture damaging functions",
                Sys.Date()
            ),
            msg
        )
        write(msg, file.path(path[["output"]][["damage"]], file.log))
    }

    if (isTRUE(simple)) {
        return(invisible(disaggregated))
    }
    if (isTRUE(retrieve)) {
        return(invisible(stats::setNames(list(disaggregated), culture)))
    }
    return(NULL)
}
