#' @title Combine damage curves
#' 
#' @description
#' `combine_damaging()` combines damage curves for specified crops or all crops
#' and for each type of damage and saves the results.
#' 
#' @param path list describing where to find useful paths.
#' @param damage.culture list of data.frames of disaggregated damage for each
#' crop to be combined. Result of `calculate.damaging()` can be used, default to
#' NULL.
#' @param hazard data.frame of hazard parameters. Should have the same number of
#' rows as the damage functions in `damage.culture`. If NULL, it will be loaded
#' from `file.path(path[["output"]][["damage"]], "hazard.csv")`.
#' Default to NULL. 
#' @param weight character indicating the base name weight method. If NULL, 
#' no weights are used and damage functions are `cbind()` in one data.frame for
#' each damage (`c("total", "yield.annual", "yield.differed","replanting", 
#' "soil", material")`). Default to NULL.
#' @param verbose logical, should the function send some messages while running.
#' @param retrieve character, which damage type should be returned. If NULL,
#' nothing is returned. Default to NULL.
#' 
#' @return `combine_damaging()` returns one data.frame for each damage type `c(
#' "total", "yield.annual", "yield.differed","replanting", "soil", material")` 
#' which contains damage curves for each crop in `names(damage.culture)`.
#'
#' @export
#'
#' @encoding UTF-8

combine_damaging = function(
    path,
    damage.culture = NULL,
    hazard = NULL,
    weight = NULL,
    verbose = TRUE,
    retrieve = NULL
) {
    if (isTRUE(verbose)) cat("Processing combinations of damaging curves...")
    begin = Sys.time()

    if (is.null(hazard)) {
        hazard = utils::read.csv2(
            file.path(path[["output"]][["damage"]], "hazard.csv")
        )[floodam.agri::HAZARD]
    }

    if (is.null(damage.culture)) {
        damage.file = list.files(
            file.path(
                path[["output"]][["damage"]],
                "sumup"
            ),
            pattern = "^damage-disaggregated_.*\\.csv$",
            full.names = TRUE
        )
        damage.culture = sapply(damage.file, utils::read.csv2, simplify = FALSE)
        names(damage.culture) = gsub(
            ".*disaggregated_(.+).csv.*",
            "\\1",
            names(damage.culture)
        )
    }
    damage.component = names(damage.culture[[1]])

    # Reading weight

    if (!is.null(weight)) {
        # Reading action.value files to know the origin of each crop yield
        action_value = find_output_files(path, "action.value.csv", "culture")
        action_value = stats::setNames(
            lapply(
                action_value,
                utils::read.csv2
            ),
            gsub(".*culture/(.+)/action.value.csv.*", "\\1", action_value)
        )
        action_value = action_value[names(damage.culture)]

        extents = unique(unlist(lapply(
            action_value,
            function(x) {result = x[x[["name"]] == "yield", "extent"]}
        )))

        # Load existing weights in coherence with used yields
        weight_all = stats::setNames(mapply(
            extent = extents,
            find_weight_files,
            MoreArgs = list(
                path = path,
                name = sprintf("weight.floodam.%s.csv", weight)
            ),
            SIMPLIFY = FALSE
        ), extents)

        # Testing if loaded weights files are coherent with the yields used

        test = unlist(lapply(weight_all, function(x) length(x) == 0))
        
        if (any(test)) {
            message = paste(
                sprintf("Yields from '%s'", names(test[test])),
                "are used but",
                sprintf("'weight.floodam.%s.csv'", weight),
                "is missing in the",
                sprintf("'%s' folder.", names(test[test]))
            )
            stop(message)
        }

        weight_all = lapply(weight_all, load_typology)

        output = combine_weight(
            weight_all,
            action_value,
            damage.culture,
            damage.component
        )
    } else {
        output = sapply(
            damage.component,
            function(x, damage.culture) {
                as.data.frame(
                    lapply(
                        damage.culture,
                        `[[`,
                        x
                    )
                )
            },
            damage.culture,
            simplify = FALSE
        )
    }

    lapply(
        names(output),
        function(x) {
            utils::write.csv2(
                cbind(hazard, output[[x]]),
                file.path(
                    path[["output"]][["combine"]],
                    if (!is.null(weight)) {
                        sprintf("%s_%s.csv", weight, x)
                    } else {
                        sprintf("%s.csv", x)
                    }
                ),
                row.names = FALSE
            )
        }
    )

    elapsed = Sys.time() - begin
    if (isTRUE(verbose)) cat("\t... Finished in", elapsed, units(elapsed), "\n")
    if (isFALSE(is.null(retrieve))) {
        return(invisible(cbind(hazard, output[[retrieve]])))
    }
}

combine_damage = function(m, w, ...) {apply(t(m) * w, 2, sum, ...)}

combine_weight = function(
    weight_all,
    action_value,
    damage.culture,
    damage.component
) {
    action_value = do.call("rbind", action_value)
    action_value = action_value[
        action_value[["name"]] == "yield", 
        c("culture", "extent")
    ]
    action_value = split(action_value, action_value[["extent"]])
    
    # Removing useless weights
    weight_all[!names(weight_all) %in% names(action_value)] = NULL

    for (i in names(action_value)) {
        weight_all[[i]] = weight_all[[i]][
            row.names(weight_all[[i]]) %in% action_value[[i]][["culture"]],
            , drop = FALSE
        ]
    }

    # Rbind data.frames of weights even if different columns
    allnames = unique(unlist(lapply(weight_all, colnames)))
    weight_all = lapply(weight_all, rbind_list, allnames)
    
    weight_all = do.call("rbind", weight_all)
    selection = colSums(weight_all, na.rm = TRUE) > 0
    weight_all = weight_all[, selection, drop = FALSE]

    # Order rows of weight alphabetically
    weight_all = weight_all[names(damage.culture), , drop = FALSE]

    damage.culture = lapply(
        damage.component,
        function(x) {
            as.matrix(as.data.frame(sapply(damage.culture, `[`, x)))
        }
    )
    names(damage.culture) = damage.component

    output = lapply(
        damage.culture,
        function(x){
            apply(as.data.frame(weight_all), 2, combine_damage, m = x, na.rm = TRUE)
        }
    )
    names(output) = names(damage.culture)
    return(output)
}

rbind_list = function(x, allnames, type = "matrix") {
     x = as.data.frame(x, check.names = FALSE)
     if (type == "matrix") {
        result = as.matrix(data.frame(
            c(x, sapply(setdiff(allnames, names(x)), function(y) NA)),
            row.names = row.names(x),
            check.names = FALSE
        ))
        return(result)
     }
     if (type == "data.frame") {
        result = data.frame(
            c(x, sapply(setdiff(allnames, names(x)), function(y) NA)),
            row.names = row.names(x),
            check.names = FALSE
        )
        return(result)
     }
}
