#' @title Create input and output paths
#' 
#' @description
#' `init_path()` creates paths that will be used to find input data and save 
#' outputs.
#'
#' @param input named character vector indicating where to find inputs. This
#' vector defines the path where to look for files in priority. If nothing is
#' specified, paths will be set to `system.file("extdata", "default", package = 
#' "floodam.agri")`.
#' @param output character, path for output. Default to `tempdir()`.
#' @param default logical, should "default" path be created. If TRUE, "default"
#' path will be set to `system.file("extdata", "default", package = 
#' "floodam.agri")`. Default to TRUE.
#' @param classification character giving the path to the folder which contains
#' classification references. If NULL, no path is created. Default to NULL.
#' 
#' @details 
#' `input` parameter:
#' The order of each element in `input` is important as it will define the order
#' in which inputs will be searched in `path[["extent"]]`.
#'
#' @return list of character, paths created.
#'
#' @examples
#'
#' path = init_path()
#'
#' # Unnamed input vector
#' input = c(
#'     system.file(
#'         "extdata",
#'         "example",
#'         "local",
#'         "winery",
#'         package = "floodam.agri"
#'     ),
#'     system.file(
#'         "extdata",
#'         "regional",
#'         "Occitanie",
#'         "2016",
#'         package = "floodam.agri"
#'     ),
#'     system.file("extdata", "national", package = "floodam.agri")
#' )
#' path = init_path(input)
#' print(path[["extent"]])
#' 
#' path = init_path(input, default = FALSE)
#' print(path[["extent"]])
#' 
#' names(input) = c("local", "regional", "national")
#' path = init_path(input)
#' print(path[["extent"]])
#'
#' @export
#'
#' @encoding UTF-8

init_path = function(
    input,
    output = tempdir(),
    default = TRUE,
    classification = NULL
) {
    path = list()

    # Default paths
    if (!missing(input)) {
        if (
            is.null(names(input)) ||
            any(
                duplicated(
                    c(names(input), c("default", "extent", "output", ""))
                )
            )
        ) {
            name = sprintf("user_%02d", seq_along(input))
        } else {
            name = names(input)
        }

        path = lapply(input, init_user_path)
        names(path) = name
        path = path[!sapply(path, is.null)]
    }

    if (isTRUE(default)) path[["default"]] = init_user_path()
    
    # Verifying that there is something in path
    if (length(path) == 0) {
        msg = sprintf(
            "Nothing found for the paths in \n%s", paste(input, collapse = "\n")
        )
        stop(msg)
    }
    
    # Declaring extent names
    path[["extent"]] = names(path)
    
    # Verifying that there is at least one stat folder in extents
    test = all(unlist(lapply(
        path[["extent"]],
        function(x) {is.null(path[[x]][["stat"]])}
    )))
    if (test) {
        msg = paste("No stat folder found in declared extents. Check inputs.")
        stop(msg)
    }

    # Declaring path where to find classification files
    if (!is.null(classification)) {
        path[["classification"]] = classification
    }

    # Declaring and creating outputs
    path[["output"]] = list(
        culture = file.path(output, "culture"),
        damage = file.path(output, "damage"),
        combine = file.path(output, "combine"),
        figure = file.path(output, "figure")
    )

    # Adding output folders for weight files if needed
    test = !is.null(
        find_data_path(path, "guideline.csv", "typology", all = TRUE)
    )
    if (test) {
        # Preparing extents for which an output typology folder is needed 
        selection = path[["extent"]] %in% names(find_data_path(
            path,
            name = "guideline.csv",
            field = "typology",
            all = TRUE
        ))
        path[["output"]][["typology"]] = stats::setNames(
            as.list(file.path(output, "typology", path[["extent"]][selection])),
            path[["extent"]][selection]
        )
    }
    
    lapply(
        unlist(path[["output"]]),
        dir.create,
        showWarnings = FALSE,
        recursive = TRUE
    )

    return(path)
}

# Create a list of paths for user. If path is not given, then it is set to
# local location of the library itself.
init_user_path = function(
    path = system.file("extdata", "default", package = "floodam.agri"),
    field = c("combine", "damagement", "hazard", "stat", "typology")
) {
    result = lapply(field, dir, path = path, full.names = TRUE)
    names(result) = field
    result = result[lengths(result) > 0]
    if (length(result) == 0) return(NULL)
    result
}

find_data_path = function(path, name, field, all = FALSE) {
    name = sprintf("^%s$", name)
    extent = sapply(path[path[["extent"]]], `[[`, field)
    result = unlist(
        sapply(
            extent[lengths(extent) != 0],
            list.files,
            pattern = name,
            recursive = TRUE,
            include.dirs = TRUE,
            full.names = TRUE
        )
    )
    result = result[lengths(result) > 0]
    if (length(result) == 0) return(NULL)
    if (isFALSE(all)) return(result[1])
    result
}

find_file_path = function(path, name, field, all = FALSE) {
    name = sprintf("%s*", name)
    extent = sapply(path[path[["extent"]]], `[[`, field)
    result = unlist(
        sapply(
            extent[lengths(extent) != 0],
            list.files,
            pattern = name,
            recursive = TRUE,
            include.dirs = TRUE,
            full.names = TRUE
        )
    )
    result = result[lengths(result) > 0]
    if (length(result) == 0) return(NULL)
    if (isFALSE(all)) return(result[1])
    result
}

find_output_files = function(path, name, field) {
    name = sprintf("^%s$", name)
    list.files(
        path[["output"]][[field]],
        pattern = name,
        recursive = TRUE,
        full.names = TRUE
    )
}

find_weight_files = function(path, name, extent) {
    name = sprintf("^%s$", name)
    if (is.null(path[["output"]][["typology"]][[extent]])) return(NULL)
    list.files(
        path[["output"]][["typology"]][[extent]],
        pattern = name,
        recursive = TRUE,
        full.names = TRUE
    )
}

list_hazard_files = function(path, name, field, all = FALSE) {
    extent = sapply(path[path[["extent"]]], `[[`, field)
    result = lapply(
                file.path(extent[lengths(extent) != 0],name),
                list.files,
                recursive = TRUE,
                include.dirs = TRUE,
                full.names = TRUE
            )
    result = result[lengths(result) == 3] # there are 3 hazard resolution files
    if (length(result) == 0) return(NULL)
    if (isFALSE(all)) return(unlist(result[1]))
    unlist(result)
}
