#' @title Make damage curve
#'
#' @description
#' `make_curve()` makes the damage curve, for a crop. This function is called in
#' `calculate_damaging()`.
#' 
#' @param damaging.core data.frame, damage function for each type of phenomena
#' impacting a crop. See
#' `inst/extdata/default/damagement/<crop>/damaging.core.csv`.
#' @param damaging.complement data.frame, complementary damaging information
#' for a crop. See
#' `inst/extdata/default/damagement/<crop>/damaging.complement.csv`.
#' @param calendar  data.frame, crop calendar.
#' @param level list giving levels for hazard parameters (current, duration, 
#' height, week), default to floodam.agri::LEVEL.
#' @param combine list of combine methods for each type of phenomena used to
#' group damaging function.
#' @param current data.frame, correspondance between velocity qualitative 
#' classes and their numeric value. Default to floodam.agri::CURRENT.
#' @param type character, should output be of class data.frame or a list,
#' default to "data.frame".
#'
#' @return 
#' if `type` is "data.frame", `make_curve()` returns a data.frame
#' 
#' if `type` is "list", `make_curve()` returns a list of 4D-arrays.
# - each element of the list corresponds to a type of damaging
# - each element is 4D-array giving damaging for any combination of the level
#'
#' @export
#'
#' @encoding UTF-8

make_curve = function(
    damaging.core,
    damaging.complement,
    calendar,
    level = floodam.agri::LEVEL,
    combine,
    current = floodam.agri::CURRENT,
    type = "data.frame"
) {
    damaging.complement = fit_complement(damaging.complement, calendar)
    damaging.type = paste(
        as.character(damaging.complement[["z"]]),
        gsub(" ", "_", as.character(damaging.complement[["component"]])),
        sep = "."
    )

    # Extrapolation of each present curve for levels on height and duration
    result = lapply(
        rownames(damaging.complement),
        function(x) {
             array_hd(
                level[["height"]],
                level[["duration"]],
                damaging.core[damaging.core[["curve"]] == x, ]
            )
        }
    )
    names(result) = rownames(damaging.complement)

    # Integration of levels on current
    result = array_current(
        result,
        current = current,
        current_start = damaging.complement[["current"]],
        level = level[["current"]]
    )

    # Integration of levels on week
    result = array_week(
        result,
        damaging.complement[, c("start.week","end.week")],
        level[["week"]],
        damaging.type,
        combine
    )

    if (type == "list") return(invisible(result))
    return(invisible(listofarray_to_df(result)))
}

#' @title Fit damaging.complement to hazard parameters
#' 
#' @description
#' `fit_complement()` transforms format of a crop damaging.complement data.frame
#' to have numeric values instead of qualitative physiological phases and
#' velocity classes. Called in `make_curve()`.
#' 
#' @param damaging.complement data.frame, complementary damaging information
#' for a crop. See
#' `inst/extdata/default/damagement/<crop>/damaging.complement.csv`.
#' @param calendar data.frame, crop calendar.
#' @param current data.frame, correspondance between velocity qualitative 
#' classes and their numeric value. Default to floodam.agri::CURRENT.
#' 
#' @return data.frame, transformed damaging.complement data.frame in coherence
#' with needed format for `make_curve()`.
#'
#' @encoding UTF-8
#' 

fit_complement = function(
    damaging.complement,
    calendar, 
    current = floodam.agri::CURRENT
) {
    # Checking phase names in `damaging.complement` and `calendar` match.
    selection = unique(c(
        damaging.complement[["start.phase"]],
        damaging.complement[["end.phase"]]
    ))
    test = selection %in% row.names(calendar)
    if (!all(test)) {
        msg = sprintf(
            "Phases of damaging.complement can't be found in calendar, see: %s",
            paste(selection[!test], collapse = ", ")
        )
        stop(msg)
    }

    # Adding start.week & end.week, numeric value corresponding to start.phase &
    # end.phase
    damaging.complement[["start.week"]] = calendar[
        as.character(damaging.complement[["start.phase"]]),
        "start.week"
    ]
    damaging.complement[["end.week"]] = calendar[
        as.character(damaging.complement[["end.phase"]]),
        "end.week"
    ]

    # Checking current in `damaging.complement` and `current` match.
    selection = unique(damaging.complement[["current"]])
    test = selection %in% row.names(current)
    if (!all(test)) {
        msg = sprintf(
            paste(
                "'current' values of damaging.complement are not in current`,",
                "see: %s"
            ),
            paste(selection[!test], collapse = ", ")
        )
        stop(msg)
    }

    # Converting current in numeric value
    damaging.complement[["current"]] = current[
        as.character(damaging.complement[["current"]]),
        "value"
    ]

    # Adding row names corresponding to curve (unique)
    rownames(damaging.complement) = as.character(damaging.complement[["curve"]])

    selection = names(damaging.complement) %in% c("curve", "start.phase", "end.phase")
    return(damaging.complement[!selection])
}

#' @title Make array current
#' 
#' @description
#' `array_current()` transforms a list of 2D-arrays on a list of 3D-arrays
#' by taking into account information on current.
#' Called in `make_curve()`.
#' 
#' @param x list of 2D-arrays (height, duration).
#' @param current data.frame, correspondance between velocity qualitative 
#' classes and their numeric value. Default to floodam.agri::CURRENT.
#' @param current_start parameter of the damaging curve giving what is the
#' starting level of current for which the curve is valid.
#' @param level character, levels of current to be included.
#' 
#' @return list of 3D-arrays (height, duration, current)
#'
#' @encoding UTF-8
#' 

array_current = function(
    x,
    current = floodam.agri::CURRENT,
    current_start,
    level
) {
    fun = function(x, current_start, current) {
        r = array(
            0,
            dim = c(dim(x), 
            nrow(current)), 
            dimnames = c(
                dimnames(x),
                current = list(current[, "value"])
            )
        )
        selection = current[, "value"] >= current_start
        r[, , selection] = x
        return(r)
    }

    result = mapply(
        fun,
        x,
        current_start,
        SIMPLIFY = FALSE,
        MoreArgs = list(current = current)
    )
    result = lapply(
        result,
        function(x) {x[, , level, drop = FALSE]}
    )
    return(result)
}