#' @title Plot aggregated damage
#' 
#' @description
#' `plot_aggregated_damage()` makes a plot of agricultural damage. It can plot
#' any damage function that has been aggregated with `aggregate_damage()`.
#'
#' @param path list describing where to find useful paths.
#' @param damage character, type of damage to be plotted. If NULL,
#' "total.aggregate.csv" is selected. Default to NULL.
#' @param current character, current to be plotted. Default to "light".
#' @param duration character, duration to be plotted. Default to "c("short",
#' "long")".
#' @param week character, periods to be plotted. If NULL, all periods will be 
#' plotted. Default to NULL.
#' @param col character, named or unnamed vector of colors for plot lines. Must 
#' be at least of the same length as week. If col is unnamed, the attribution of
#' colors to week will be made by default, you must set names of col accordingly
#' to values of week if you want to control the colors for each week. If NULL,
#' default colors will be picked from the library("RColorBrewer"). Default to 
#' NULL.
#' @param culture character, name of crops to be plotted. If set to NULL, all
#' crops available in the provided damage function will be plotted. Default to
#' NULL.
#' @param output_prefix character, prefix to give to output file. Default to
#' NULL
#' @param output_format character, choice for the output format if saving is
#' desired. If NULL or "none", plot will not be saved but will be shown. Default
#' to NULL.
#' @param language character, language for plot labels. Must be chosen from one
#' of c("EN", "FR"). Default to "EN".
#' @param hazard character, name of hazard parameters.
#' Default to floodam.agri::HAZARD.
#' @param verbose logical, should the function send some messages while running.
#' @param legend character, choice for the legend format. If NULL or "none", 
#' legend will not be created.
#' @param include_legend character, position of legend inside the plot. `legend`
#' must be set to "include" for `include_legend` to be used.
#' 
#' @return damage curve or damage curves for aggregated damage functions. It
#' can be simply shown or saved in the `output_format` format in the output
#' directory from `path[["output"]][["figure"]]`
#'
#' @export
#'
#' @details
#' 
#' By default, `plot_aggregated_damage()` will return a title for the plot which
#' includes the name of the crop that is plotted. This title is given by a 
#' data.frame giving the correspondance between the floodam crop name and the
#' crop name to be shown on the plot. To customize the crop names as you desire,
#' a csv file named `title.culture.csv` should be added to your `typology`
#' folder. The first column should be the crop name used for building the damage
#' functions, the second column should be the name to be plotted.
#' 
#' @encoding UTF-8

plot_aggregated_damage = function(
    path,
    damage = "total.aggregate",
    current = "light",
    duration = c("short", "long"),
    week = NULL,
    col = NULL,
    culture = NULL,
    output_prefix = NULL,
    output_format = c("none", "pdf", "png", "jpg", "svg"),
    language = "EN",
    hazard = floodam.agri::HAZARD,
    verbose = TRUE,
    legend = c("none", "include", "horizontal", "vertical"),
    include_legend = c("topleft", "bottomright", "bottom", "bottomleft", "left",
    "top", "topright", "right")
) {
    # Checking function arguments
    output_format = match.arg(output_format)
    legend = match.arg(legend)
    include_legend = match.arg(include_legend)
    dataset = utils::read.csv2(
        list.files(
            file.path(path[["output"]][["combine"]]),
            pattern = damage,
            full.names = TRUE
        ),
        check.names = FALSE
    )
    
    if (is.null(culture)) {
        title_culture = utils::read.csv2(
            find_file_path(path, "title.culture.csv", "typology", all = FALSE)
        )
        title_culture = title_culture[
            title_culture[["code"]] %in% colnames(dataset[-c(1:4)]), ]
    } else {
        title_culture = utils::read.csv2(
            find_file_path(path, "title.culture.csv", "typology", all = FALSE)
        )
        title_culture = title_culture[title_culture[["code"]] %in% culture, ]
    }

    if (is.null(current)) {current = unique(dataset[["current"]])}
    if (is.null(duration)) {duration = unique(dataset[["duration"]])}
    if (is.null(week)) {week = unique(dataset[["week"]])}
    
    # remove useless hazard parameters
    selection = dataset[["current"]] %in% current &
        dataset[["duration"]] %in% duration &
        dataset[["week"]] %in% week
    dataset = dataset[selection,]

    lty = c("solid", "dashed", "dotted", "dotdash", "longdash", "twodash")[
        seq(1:length(duration))
    ]

    # Dealing with colors
    if (is.null(col)) {
        col = stats::setNames(
            c(
                c(
                    "chartreuse3",
                    "gold2",
                    "darkorange4",
                    "deepskyblue1"
                ),
                RColorBrewer::brewer.pal(9, "Spectral")
            )[1:length(week)],
            week
        )
    }
    if (length(col) < length(week)) {
        msg = sprintf(
            "length(col) must be equal or superior to %s (length(week))",
            length(week)
        )
        stop(msg)
    }
    if (length(col) > length(week)) {
        col = col[1:length(week)]
    }
    if (is.null(names(col))) col = stats::setNames(col, week)
    if (any(!week %in% names(col))) {
        msg = "Missing week names in 'col'. Verify the names given to col."
        stop(msg)
    }

    plan_experiment = stats::setNames(lapply(
        current,
        function(x, duration, week, col, lty, lwd = 2) {
            plan_experiment = data.frame(
                current = x,
                duration = rep(duration, each = length(week)),
                week = rep(week, length(duration)),
                col = rep(col[week], length(duration)),
                lty = rep(lty, each = length(week)),
                lwd = rep(lwd, length(x))
            )
        },
        duration,
        week,
        col,
        lty
    ), current)

    if (verbose) {
        if (output_format == "none" & nrow(title_culture) > 1) {
            message(
                "Only 1 crop can be viewed at a time.\n",
                "To view other crop damage functions, you can:\n",
                "   - set `culture` to a specific crop\n",
                "   - specify `output_name` and set `culture` to NULL ",
                "(export plots for all crops)" 
            )
        }
    }

    lapply(
        plan_experiment,
        function(
            x,
            title_culture,
            path,
            dataset,
            output_prefix,
            output_format, 
            language,
            legend,
            include_legend
        ) {
            apply(
                title_culture,
                1,
                plot_damage,
                path = path,
                d = dataset,
                pe = x,
                output_prefix = output_prefix,
                output_format = output_format,
                current = unique(x[["current"]]),
                language = language,
                legend = legend,
                include_legend = include_legend
            )
        },
        title_culture,
        path,
        dataset,
        output_prefix,
        output_format,
        language,
        legend,
        include_legend
    )

    return(invisible())
}

lines_sel = function(x, d, title_culture) {
    selection =
        d[["current"]] == x[1] &
        d[["duration"]] == x[2] &
        d[["week"]] == x[3]

    graphics::lines(
        x = d[selection, "height"],
        y = d[selection, title_culture],
        type = "l",
        col = x[4],
        lty = x[5],
        lwd = as.numeric(x[6]),
        xlim = c(0, max(d["height"])),
        ylim = c(0, max(d[title_culture]))
    )
}

plot_damage = function(
    title_culture,
    path,
    d,
    pe,
    output_prefix,
    output_format,
    current,
    hazard = floodam.agri::HAZARD,
    language,
    legend,
    include_legend
) {
    if (language == "FR") {
        xlab = "Hauteur (cm)"
        ylab = "Dommages (euros/ha)"
    }

    if (language == "EN") {
        xlab = "Height (cm)"
        ylab = "Damage (euros/ha)"
    }

    if (output_format != "none") {
        name = if (is.null(output_prefix)) {
            file.path(
                path[["output"]][["figure"]],
                sprintf(
                    "%s-%s-%s.%s",
                    gsub(" ", "_", title_culture[1]),
                    current,
                    language,
                    output_format
                )
            )
        } else {
            file.path(
                path[["output"]][["figure"]],
                sprintf(
                    "%s-%s-%s-%s.%s",
                    output_prefix,
                    gsub(" ", "_", title_culture[1]),
                    current,
                    language,
                    output_format
                )
            )
        }

        width = 18
        height = 18
        switch(
            EXPR = tolower(tools::file_ext(name)), 
            pdf = grDevices::cairo_pdf(
                filename = name,
                width = width/2.54,
                height = height/2.54
            ),
            png = grDevices::png(
                filename = name, 
                width = width,
                height = height,
                units = "cm",
                res = 144
            ),
            jpg = grDevices::jpeg(
                filename = name, 
                width = width,
                height = height,
                units = "cm", 
                res = 144
            ),
            svg = grDevices::svg(
                filename = name,
                width = width/2.54,
                height = height/2.54
            ),
            stop(sprintf(
                "%s not recognized", 
                tolower(tools::file_ext(name))
            ))
        )
        on.exit(
            message("Output created at: ", name),
            add = TRUE,
            after = TRUE
        )
        on.exit(grDevices::dev.off(), add = TRUE, after = TRUE)
    }

    plot(
        x = NULL,
        main = paste(
            title_culture[2],
            sprintf("%s current", unique(pe[["current"]])),
            sep = " - "
        ),
        xlab = xlab,
        ylab = ylab,
        xlim = c(0, max(d["height"])),
        ylim = c(0, max(d[d[["current"]] == current, title_culture[[1]]]))
    )

    apply(
        pe,
        1,
        lines_sel,
        d = d[c(hazard, title_culture[[1]])],
        title_culture[[1]]
    )
    
    if (legend == "horizontal") horiz = TRUE else horiz = FALSE
    if (legend == "include") {
        legend(
            include_legend,
            legend = c(
                unique(pe[["week"]]),
                unique(pe[["duration"]])
            ),
            fill = c(
                unique(pe[["col"]]),
                rep(0, length(unique(pe[["lty"]])))
            ),
            border = c(
                rep("black", length(unique(pe[["col"]]))),
                rep(NA, length(unique(pe[["lty"]])))
            ),
            lty = c(
                rep(NA, length(unique(pe[["col"]]))),
                unique(pe[["lty"]])
            ),
            x.intersp = c(
                rep(-1, length(unique(pe[["col"]]))),
                rep(0.5, length(unique(pe[["lty"]])))
            ),
            horiz = horiz,
            bty = "n"
        )
    }
}