#' @title Plot crop calendar
#' 
#' @description
#' `plot_crop_calendar()` makes a plot of the calendar of one or more crops. It*
#' can plot any crop calendar prepared with `prepare_calendar()`.
#'
#' @param path list describing where to find useful paths.
#' @param col character, named or unnamed vector of colors for plot. Must be at
#' least of the same length as crops phases. If col is unnamed, the attribution of
#' colors to crop phases will be made by default, you must set names of col
#' accordingly to values of crop phases if you want to control the colors for
#' each phase. If NULL, default colors will be picked from the
#' library("RColorBrewer"). Default to NULL.
#' @param culture character, name of crops to be plotted. If set to NULL, all
#' crops prepared with `prepare_culture()` and `prepare_calendar()` will be 
#' plotted. Default to NULL.
#' @param title_culture logical, should shown crop names on plot be modified. If
#' set to TRUE, `plot_damage_function()` will look for the first 
#' available `title.culture.csv` in `path[[<extent>]][["typology"]]`. Default to
#' FALSE.
#' @param language character, language for plot labels. Must be chosen from one
#' of c("EN", "FR"). Default to "EN".
#' @param legend logical, should there be a legend or not. Default set to TRUE
#' @param reference character, culture to place on top of graphic to compare
#' with others. If set to NULL, order is set alphabetically. Default to NULL.
#' @param order_phase character, phase used to sort calendars on graph. If set 
#' to NULL, the order is set alphabetically except if a `reference` is defined.
#' Default to NULL.
#' @param output_prefix character, prefix to give to output file. Default to
#' @param output_format character, choice for the output format if saving is
#' desired. If NULL or "none", plot will not be saved but will be shown. Default
#' to NULL.
#' @param verbose logical, should the function send some messages while running.
#' 
#' @return Plot of one or more crop calendar. It can be simply shown or saved 
#' in the `output_format` format in the output directory from 
#' `path[["output"]][["figure"]]`
#'
#' @export
#'
#' @encoding UTF-8

plot_crop_calendar  = function(
    path,
    culture = NULL,
    title_culture = FALSE,
    col = NULL,
    language = "EN",
    legend = TRUE,
    reference = NULL,
    order_phase = NULL,
    output_prefix = NULL,
    output_format = c("none", "pdf", "png", "jpg", "svg"),
    verbose = TRUE
) {
    output_format = match.arg(output_format)
    if (is.null(culture)) {
        culture = load_typo(path)[["type2"]]
    }
    if (isTRUE(title_culture)) {
        title_culture = utils::read.csv2(
            find_file_path(path, "title.culture.csv", "typology", all = FALSE)
        )
        title_culture = title_culture[title_culture[["code"]] %in% culture, ]
    }

    # Preparing plot data
    plot_data = sort(find_output_files(path, "calendar.csv", "culture"))
    plot_data = stats::setNames(
        lapply(plot_data, utils::read.csv2),
        sub(".*culture/(.*?)/calendar.csv.*", "\\1", plot_data)
    )
    plot_data = plot_data[culture]
    for (i in names(plot_data)) {plot_data[[i]][["culture"]] = i}
    phases = unique(lapply(plot_data, '[[', "phase"))
    
    plot_data = lapply(
        phases,
        function(x, plot_data) {
            selection = lapply(plot_data, function(y) {
                identical(y[["phase"]], x)
            })
            result = plot_data[unlist(selection)]
            result = Reduce(
                rbind,
                result
            )
        },
        plot_data
    )
    
    if (!is.null(col)) {
        min_col = max(
            sapply(plot_data, function(x) {length(unique(x[["phase"]]))})
        )
        if (length(col) < min_col) {
            msg = "length(col) is inferior to the maximum number of crop phases"
            msg2 = sprintf(
                "length(col) should be at least of length: %s",
                min_col
            )
            stop(paste(msg, msg2, sep = "\n"))
        }
    }
    
    if (length(plot_data) > 1) {
        if (!is.null(order_phase)) {
            message(
                "'order_phase' is ignored due to different physiological stages."
            )
        }
        if (!is.null(reference)) {
            message(
                "'reference' is ignored due to different physiological stages."
            )
        }
        if (!is.null(reference) || !is.null(order_phase)) {
            message(
                "To make sure it is not ignored, set 'culture' to crops with the same physiological stages."
            )
        }
        lapply(
            plot_data,
            make_calendar_plot,
            path = path,
            col = col,
            culture = culture,
            title_culture = title_culture,
            order_phase = order_phase,
            reference = reference,
            language = language,
            output_format = output_format,
            output_prefix = output_prefix
        )
    } else {
        lapply(
            plot_data,
            make_calendar_plot,
            path = path,
            col = col,
            culture = culture,
            title_culture = title_culture,
            order_phase = order_phase,
            reference = reference,
            language = language,
            output_format = output_format,
            output_prefix = output_prefix
        )
    }
    return(invisible())
}

calendar_segments = function(
    x0,
    x1,
    positions,
    col,
    lty,
    lwd = 10,
    offset = 0.3
) {
    graphics::segments(
        x0 = x0 + offset,
        y0 = positions,
        x1 = x1 - offset,
        col = col,
        lwd = lwd,
        lty = lty
    )
}

make_calendar_plot = function(
    plot_data,
    path,
    col = NULL,
    culture,
    title_culture,
    order_phase,
    reference,
    language,
    output_format,
    output_prefix
) {
    plot_data[["culture"]] = factor(plot_data[["culture"]])
    plot_data = stats::reshape(
        plot_data,
        timevar = "phase",
        idvar = "culture",
        direction = "wide"
    )

    names(plot_data) = gsub("start.week", "", names(plot_data))
    names(plot_data) = gsub("\\.", "", names(plot_data))
    names(plot_data) = gsub(" ", "_", names(plot_data))

    if (!is.null(order_phase)) {
        if (order_phase %in% colnames(plot_data)) {
            plot_data = plot_data[
                order(plot_data[[order_phase]],
                decreasing = FALSE),
            ]
        }
        if (!is.null(reference)) {
            if (reference %in% plot_data[["culture"]]) {
                selection = plot_data[["culture"]] == reference
                plot_data = rbind(
                    plot_data[selection, ],
                    plot_data[!selection, ]
                )
            }
        }
    }

    if (!is.null(reference) && is.null(order_phase)) {
        if (reference %in% plot_data[["culture"]]) {
            selection = plot_data[["culture"]] == reference
            plot_data = rbind(
                plot_data[selection, ],
                plot_data[!selection, ]
            )
        }
    }

    # Dealing graph parameters
    if (is.null(col)) {
        col = grDevices::colorRampPalette(
            RColorBrewer::brewer.pal(9, "Spectral")
        )(ncol(plot_data)-1)
        names(col) = names(plot_data[2:ncol(plot_data)])
        col[names(col) %in% c("nude_parcel", "nothing1", "nothing2")] = "grey"
    }

    if (length(col) > ncol(plot_data)-1) {
        col = col[1:ncol(plot_data)-1]
    }

    if (is.null(names(col))) {
        names(col) = names(plot_data[2:ncol(plot_data)])
    }

    lty = stats::setNames(
        rep(
            "solid",
            ncol(plot_data)-1
        ),
        names(plot_data[2:ncol(plot_data)])
    )
        
    width = 16
    height = 8
    if (output_format != "none") {
        name = if (is.null(output_prefix)) {
            file.path(
                path[["output"]][["figure"]],
                sprintf(
                    "calendar_%s-%s.%s",
                    plot_data[1, "culture"],
                    language,
                    output_format
                )
            )
        } else {
            file.path(
                path[["output"]][["figure"]],
                sprintf(
                    "calendar-%s-%s-%s.%s",
                    output_prefix,
                    plot_data[1, "culture"],
                    language,
                    output_format
                )
            )
        }

        switch(
            EXPR = tolower(tools::file_ext(name)), 
            pdf = grDevices::cairo_pdf(
                filename = name,
                width = width,
                height = height
            ),
            png = grDevices::png(
                filename = name, 
                width = width*2.54,
                height = height*2.54,
                units = "cm",
                res = 144
            ),
            jpg = grDevices::jpeg(
                filename = name, 
                width = width*2.54,
                height = height*2.54,
                units = "cm", 
                res = 144
            ),
            svg = grDevices::svg(
                filename = name,
                width = width,
                height = height
            ),
            stop(sprintf(
                "%s not recognized", 
                tolower(tools::file_ext(name))
            ))
        )
        on.exit(
            message("Output created at: ", name),
            add = TRUE,
            after = TRUE
        )
        on.exit(grDevices::dev.off(), add = TRUE, after = TRUE)
    }
    
    graphics::par(mar = c(7, 11, 5.1, 1))
    y_height = nrow(plot_data)*10
    plot(
        x = 1,                 
        xlab = NA, 
        ylab = NA,
        xlim = c(1, 52), 
        ylim = c(1, y_height),
        type = "n",
        xaxt = "n",
        yaxt = "n",
        cex.lab = 2
    )
    week_axis(factor = 0.12, language = language)
    
    graphics::mtext(
        "Culture",
        side = 2,
        line = 11,
        cex = 2
    )

    positions = rev(seq(5, y_height, 10))
    graphics::axis(
        2,
        at = positions,
        labels = if (!isFALSE(title_culture)) {
            title_culture[match(
                plot_data[["culture"]],
                title_culture[["code"]]
            ), "titre"]
        } else {plot_data[["culture"]]},
        # tck = 1,
        # lty = 2,
        # col = "grey",
        las = 2,
        cex.axis = 1.2
    )
    
    for (i in 2:ncol(plot_data)) {
        col_i = col[colnames(plot_data[i])]
        lty_i = lty[colnames(plot_data[i])]
        if (i == ncol(plot_data)) {
            if (all(plot_data[[i]] < plot_data[[2]])) {
                x0 = plot_data[[i]]
                x1 = plot_data[[2]]
                calendar_segments(x0, x1, positions, col_i, lty_i)
            } else {
                if (any(plot_data[[i]] < plot_data[[2]])) {
                    selection = plot_data[[i]] < plot_data[[2]]
                    x0 = plot_data[selection, i]
                    x1 = plot_data[selection, 2]
                    calendar_segments(
                        x0,
                        x1,
                        positions[selection],
                        col_i,
                        lty_i
                    )
                }
                selection = ! plot_data[[i]] < plot_data[[2]]
                x0 = plot_data[selection, i]
                calendar_segments(x0, 53, positions[selection], col_i, lty_i)
                x1 = plot_data[selection, 2]
                calendar_segments(0, x1, positions[selection], col_i, lty_i)
            }
            next
        }
        if (all(plot_data[[i]] < plot_data[[i + 1]])) {
            x0 = plot_data[[i]]
            x1 = plot_data[[i + 1]]
            calendar_segments(x0, x1, positions, col_i, lty_i)
        } else {
            if (any(plot_data[[i]] < plot_data[[i + 1]])) {
                selection = plot_data[[i]] < plot_data[[i + 1]]
                x0 = plot_data[selection, i]
                x1 = plot_data[selection, i + 1]
                calendar_segments(x0, x1, positions[selection], col_i, lty_i)
            }
            selection = ! plot_data[[i]] < plot_data[[i + 1]]
            x0 = plot_data[selection, i]
            calendar_segments(x0, 53, positions[selection], col_i, lty_i)
            x1 = plot_data[selection, i + 1]
            calendar_segments(0, x1, positions[selection], col_i, lty_i)
        }
    }

    graphics::legend(
        x = "top",
        legend = names(col),
        col = col,
        lty = 1,
        lwd = 10,
        ncol = ceiling(length(names(col))/2),
        inset = c(0, -0.15),
        x.intersp = 1,
        xpd = TRUE,
        cex = 1.1
    )
}
