#' @title Plot damage functions
#' 
#' @description
#' `plot_damage_function()` makes a plot of agricultural damage. It can
#' plot any damage function that has been combined with `combine_damaging()`.
#'
#' @param path list describing where to find useful paths.
#' @param damage character, type of damage to be plotted. If NULL,
#' "total.csv" is selected. Default to NULL.
#' @param damage_path character, path to folder where to find damage curves. If
#' NULL, damage_path is set to `path[["output"]][["combine"]]`. Default to NULL.
#' @param current numeric, current to be plotted. Default to NULL.
#' @param duration numeric, duration to be plotted. Default to NULL.
#' @param height numeric, height to be plotted. Default to NULL.
#' @param col character, named or unnamed vector of colors for plot lines. Must 
#' be at least of the same length as culture. If col is unnamed, the attribution
#' of colors to culture will be made by default, you must set names of col 
#' accordingly to values of culture if you want to control the colors for each
#' crop. If NULL, default colors will be picked from the 
#' library("RColorBrewer"). Default to NULL.
#' @param culture character, name of crops to be plotted. If set to NULL, all
#' crops available in the provided damage function will be plotted. Default to
#' NULL.
#' @param title_culture logical, should shown crop names on plot be modified. If
#' set to TRUE, `plot_damage_function()` will look for the first 
#' available `title.culture.csv` in `path[[<extent>]][["typology"]]`. Default to
#' FALSE.
#' @param add_hazard_info logical, should there be annotations giving the 
#' hazard parameters of the plotted damage curves. Default to TRUE.
#' @param main character, text to pass as plot title. If NULL, no title is
#' added. Default to NULL.
#' @param output_name character, name to give to output file. Output format can
#' be one of c("pdf", "png", "jpg", "svg"). If NULL, the plot will not be
#' exported. Default to NULL.
#' @param language character, language for plot labels. Must be chosen from one
#' of c("EN", "FR"). Default to "EN".
#' @param verbose logical, should the function send some messages while running.
#' @param legend logical, should a legend be added to the plot. Default position
#' is given by the argument `legend_pos`.
#' @param legend_pos character, position of legend inside the plot. One of c(
#' "topleft", "top", "topright", "bottomright", "bottom", "bottomleft", "left",
#' "right", "export_horizontal", "export_vertical")
#' @param hazard character, name of hazard parameters.
#' Default to floodam.agri::HAZARD.
#' @param add logical, should the plot be added to an existing plot. Default to
#' FALSE.
#' 
#' @return damage curve or damage curves for aggregated damage functions. It
#' can be simply shown or saved in the `output_format` format in the output
#' directory from `path[["output"]][["figure"]]`
#'
#' @export
#' 
#' 
#' @encoding UTF-8

plot_damage_function = function(
    path,
    damage = "total",
    damage_path = NULL,
    current = NULL,
    duration = NULL,
    height = NULL,
    col = NULL,
    culture = NULL,
    title_culture = FALSE,
    main = NULL,
    add_hazard_info = TRUE,
    output_name = NULL,
    language = "EN",
    hazard = floodam.agri::HAZARD,
    verbose = TRUE,
    legend = TRUE,
    legend_pos = c("top", "topright", "topleft", "bottomright", "bottom",
    "bottomleft", "left", "right", "export_horizontal", "export_vertical"),
    add = FALSE
) {
    # Checking function arguments
    legend_pos = match.arg(legend_pos)

    # Setting missing arguments and reading data
    if (is.null(damage_path)) damage_path = path[["output"]][["combine"]]
    plot_data = utils::read.csv2(
        list.files(
            file.path(damage_path),
            pattern = paste0(damage, ".csv"),
            full.names = TRUE
        ),
        check.names = FALSE
    )
    if (is.null(culture)) {
        culture = names(plot_data[-which(hazard %in% colnames(plot_data))])
    }
    if (isTRUE(title_culture)) {
        title_culture = utils::read.csv2(
            find_file_path(path, "title.culture.csv", "typology", all = FALSE)
        )
        test = culture %in% title_culture[["code"]]
        if (any(!test)) {
            msg = paste(
                "Missing the following crops in title_culture",
                paste(culture[!test], collapse = "\n- "),
                sep = "\n- "
            )
            stop(msg)
        }
        title_culture = title_culture[title_culture[["code"]] %in% culture, ]
    }
    plot_data = plot_data[, c(hazard, culture)]
    if (is.null(current)) {current = unique(plot_data[["current"]])[[1]]}
    if (is.null(height)) {
        height =  unique(
            plot_data[plot_data[["height"]] > 0, "height"]
        )[[1]]
    }
    if (is.null(duration)) {
        duration = unique(
            plot_data[plot_data[["duration"]] > 0, "duration"]
        )[[1]]
    }

    selection = plot_data[["duration"]] == duration &
        plot_data[["height"]] == height &
        plot_data[["current"]] == current

    plot_data = plot_data[selection,]

    # Dealing with colors
    if (is.null(col)) {
        col = stats::setNames(
            RColorBrewer::brewer.pal(
                length(culture),
                "Paired"
            )[1:length(culture)],
            culture
        )
    }
    if (length(col) < length(culture)) {
        msg = sprintf(
            "length(col) must be equal or superior to %s (length(culture))",
            length(culture)
        )
        stop(msg)
    }
    if (length(col) > length(culture)) {
        col = col[1:length(culture)]
    }
    if (is.null(names(col))) col = stats::setNames(col, culture)
    if (any(!culture %in% names(col))) {
        msg = "Missing culture names in 'col'. Verify the names given to col."
        stop(msg)
    }

    p_width = 16
    p_height = 8
    
    if (!is.null(output_name)) {
        name = file.path(
            path[["output"]][["figure"]],
            output_name
        )
        if (add != TRUE) {
            switch(
                EXPR = tolower(tools::file_ext(name)), 
                pdf = grDevices::cairo_pdf(
                    filename = name,
                    width = p_width,
                    height = p_height
                ),
                png = grDevices::png(
                    filename = name, 
                    width = p_width*2.54,
                    height = p_height*2.54,
                    units = "cm",
                    res = 144
                ),
                jpg = grDevices::jpeg(
                    filename = name, 
                    width = p_width*2.54,
                    height = p_height*2.54,
                    units = "cm", 
                    res = 144
                ),
                svg = grDevices::svg(
                    filename = name,
                    width = p_width,
                    height = p_height
                ),
                stop(
                    sprintf("%s not recognized", tolower(tools::file_ext(name)))
                )
            )
            on.exit(grDevices::dev.off(), add = TRUE, after = TRUE)
            on.exit(
                message("Output created at: ", name),
                add = TRUE,
                after = TRUE
            )
        } else {
            on.exit(switch(
                EXPR = tolower(tools::file_ext(name)),
                pdf = grDevices::cairo_pdf(
                    filename = name,
                    width = p_width,
                    height = p_height
                ),
                png = grDevices::png(
                    filename = name, 
                    width = p_width*2.54,
                    height = p_height*2.54,
                    units = "cm",
                    res = 144
                ),
                jpg = grDevices::jpeg(
                    filename = name, 
                    width = p_width*2.54,
                    height = p_height*2.54,
                    units = "cm", 
                    res = 144
                ),
                svg = grDevices::svg(
                    filename = name,
                    width = p_width,
                    height = p_height
                ),
                stop(
                    sprintf("%s not recognized", tolower(tools::file_ext(name)))
                )
            ), add = TRUE, after = TRUE)
            on.exit(grDevices::dev.off(), add = TRUE, after = TRUE)
            on.exit(
                message("Output created at: ", name),
                add = TRUE,
                after = TRUE
            )
        }
    }

    # Plot data
    if (add != TRUE) {
        graphics::par(mar = c(7, 10, 5, 5))
        max_value = max(plot_data[culture])

        plot(
            x = 1,                 
            xlab = NA, 
            ylab = NA,
            xlim = c(0, 52), 
            ylim = c(0, max_value*1.2),
            type = "n",
            xaxt = "n",
            yaxt = "n"
        )
        week_axis(factor = 0.10, language = language)

        graphics::axis(
            2,
            at = seq(0, max_value, round_up_nice(max_value)/10),
            labels = seq(0, max_value, round_up_nice(max_value)/10),
            tck = 1,
            lty = 2,
            col = "grey",
            las = 2
        )
        ylab = {
            if (language == "EN") {"Damage amount (in \U20AC/ha)"}
            if (language == "FR") {"Montant de dommage (en \U20AC/ha)"}
        }
        graphics::mtext(
            ylab,
            side = 2,
            line = 5,
            cex = 1.2
        )
    }

    if (!is.null(main)) graphics::title(main = main, cex = 2)

    if (isTRUE(add_hazard_info)) { 
        graphics::mtext(
            sprintf(
                "duration = %s ; height = %s ; current = %s",
                duration,
                height,
                current
            ),
            side = 3,
            line = 0.2,
            cex = 1
        )
    }

    # Adding legend
    if (isTRUE(legend)) {
        legend = if (!isFALSE(title_culture)) {
            title_culture[match(
                culture,
                title_culture[["code"]]
            ), "titre"]
        } else {culture}
        if (legend_pos %in% c("export_horizontal", "export_vertical")) {
            "Exporting legend will be available soon"
        } else {
            if (length(culture) > 4) {
                legend(
                    x = legend_pos,
                    legend = legend,
                    col = col,
                    lty = 1,
                    lwd = 4,
                    ncol = length(culture)/2,
                    bg = "white"
                ) } else {
                legend(
                    x = legend_pos,
                    legend = legend,
                    col = col,
                    lty = 1,
                    lwd = 4,
                    bg = "white"
                )
            }
        }
    }

    sapply(
        culture,
        damage_lines,
        plot_data,
        col
    )
    return(invisible())
}


damage_lines = function(culture, plot_data, col) {
    graphics::lines(
        c(0, plot_data[["week"]]),
        c(
            plot_data[[culture]][length(plot_data[[culture]])],
            plot_data[[culture]]
        ),
        col = col[[culture]],
        lwd = 2
    )
}
