#' @title Plot disaggregated damage functions
#' 
#' @description
#' `plot_disaggregated_damage_function()` makes a plot of disaggregated damage
#' curves. It can plot any damage function that has been combined with
#' `combine_damaging()`.
#'
#' @param path list describing where to find useful paths.
#' @param damage_path character, path to folder where to find damage curves. If
#' NULL, damage_path is set to `path[["output"]][["combine"]]`. Default to NULL.
#' @param current numeric, current to be plotted. Default to NULL.
#' @param duration numeric, duration to be plotted. Default to NULL.
#' @param height numeric, height to be plotted. Default to NULL.
#' @param col character, named or unnamed vector of colors for plot lines. Must 
#' be at least of the same length as culture. If col is unnamed, the attribution
#' of colors to culture will be made by default, you must set names of col 
#' accordingly to values of culture if you want to control the colors for each
#' crop. If NULL, default colors will be picked from the 
#' library("RColorBrewer"). Default to NULL.
#' @param culture character, name of crops to be plotted. If set to NULL, all
#' crops available in the provided damage function will be plotted. Default to
#' NULL.
#' @param title_culture logical, should shown crop names on plot be modified. If
#' set to TRUE, `plot_damage_function()` will look for the first 
#' available `title.culture.csv` in `path[[<extent>]][["typology"]]`. Default to
#' FALSE.
#' @param add_hazard_info logical, should there be annotations giving the 
#' hazard parameters of the plotted damage curves. Default to TRUE.
#' @param output_name character, name to give to output file. Output format can
#' be one of c("pdf", "png", "jpg", "svg"). If NULL, the plot will not be
#' exported. Default to NULL.
#' @param language character, language for plot labels. Must be chosen from one
#' of c("EN", "FR"). Default to "EN".
#' @param verbose logical, should the function send some messages while running.
#' @param legend logical, should a legend be added to the plot. Default position
#' is given by the argument `legend_pos`.
#' @param legend_pos character, position of legend inside the plot. One of c(
#' "topleft", "top", "topright", "bottomright", "bottom", "bottomleft", "left",
#' "right", "export_horizontal", "export_vertical")
#' @param hazard character, name of hazard parameters.
#' Default to floodam.agri::HAZARD.
#' 
#' @return damage curve or damage curves for aggregated damage functions. It
#' can be simply shown or saved in the `output_format` format in the output
#' directory from `path[["output"]][["figure"]]`
#'
#' @export
#' 
#' @encoding UTF-8

plot_disaggregated_damage_function = function(
    path,
    damage_path = NULL,
    current = NULL,
    duration = NULL,
    height = NULL,
    col = NULL,
    culture = NULL,
    title_culture = FALSE,
    add_hazard_info = TRUE,
    output_name = NULL,
    language = "EN",
    hazard = floodam.agri::HAZARD,
    verbose = TRUE,
    legend = TRUE,
    legend_pos = c("topleft", "topright", "top", "bottomright", "bottom",
    "bottomleft", "left", "right", "export_horizontal", "export_vertical")
) {
    # Checking function arguments
    legend_pos = match.arg(legend_pos)

    # Setting missing arguments and reading data
    if (is.null(damage_path)) damage_path = path[["output"]][["combine"]]
    files = list.files(
        damage_path, pattern = ".csv", full.names = TRUE
    )
    selection = grep("*aggregate|*total", files)
    plot_data = stats::setNames(
        lapply(files[-selection], utils::read.csv2),
        sub(".*combine/","", sub(".csv","", files[-selection]))
    )
    if (is.null(culture)) {
        culture = names(
            plot_data[[1]][-which(hazard %in% colnames(plot_data[[1]]))]
        )
    }
    if (isTRUE(title_culture)) {
        title_culture = utils::read.csv2(
            find_file_path(path, "title.culture.csv", "typology", all = FALSE)
        )
        test = culture %in% title_culture[["code"]]
        if (any(!test)) {
            msg = paste(
                "Missing the following crops in title_culture",
                paste(culture[!test], collapse = "\n- "),
                sep = "\n- "
            )
            stop(msg)
        }
        title_culture = title_culture[title_culture[["code"]] %in% culture, ]
    }
    plot_data = lapply(plot_data, function(x) {x = x[, c(hazard, culture)]})

    if (is.null(current)) {current = unique(plot_data[[1]][["current"]])[[1]]}
    if (is.null(height)) {
        height =  unique(
            plot_data[[1]][plot_data[[1]][["height"]] > 0, "height"]
        )[[1]]
    }
    if (is.null(duration)) {
        duration = unique(
            plot_data[[1]][plot_data[[1]][["duration"]] > 0, "duration"]
        )[[1]]
    }

    # Subsetting to chosen flood parameters
    plot_data = lapply(plot_data, function(x) {
        selection = x[["duration"]] == duration &
            x[["height"]] == height &
            x[["current"]] == current
        x = x[selection,]
        x = x[order(x[["week"]]), ]
        return(x)
    })

    # Dealing with colors
    if (is.null(col)) {
        col = stats::setNames(scales::alpha(
            RColorBrewer::brewer.pal(
                length(names(plot_data)),
                "Paired"
            )[1:length(names(plot_data))],
            0.7),
            names(plot_data)
        )
    }
    if (length(col) < length(names(plot_data))) {
        msg = sprintf(
            "length(col) must be equal or superior to %s (n type of damage))",
            length(names(plot_data))
        )
        stop(msg)
    }
    if (length(col) > length(names(plot_data))) {
        col = col[1:length(names(plot_data))]
    }
    if (is.null(names(col))) col = stats::setNames(col, names(plot_data))
    if (any(!names(plot_data) %in% names(col))) {
        msg = "Missing damage names in 'col'. Verify the names given to col."
        stop(msg)
    }

    if (verbose) {
        if (is.null(output_name) & length(culture) > 1) {
            message(
                "Only 1 crop can be viewed at a time.\n",
                "To view other crop damage functions, you can:\n",
                "   - set `culture` to a specific crop\n",
                "   - specify `output_name` and set `culture` to NULL ",
                "(export plots for all crops)" 
            )
        }
    }

    # Plotting data
    lapply(
        culture,
        disaggregated_damage,
        path = path,
        title_culture = title_culture,
        plot_data = plot_data,
        current = current,
        duration = duration,
        height = height,
        col = col,
        add_hazard_info = add_hazard_info,
        output_name = output_name,
        language = language,
        hazard = hazard,
        verbose = verbose,
        legend = legend,
        legend_pos = legend_pos
    )
    return(invisible())
}

disaggregated_damage = function(
    culture,
    path,
    title_culture,
    plot_data,
    current,
    duration,
    height,
    col,
    add_hazard_info,
    output_name,
    language,
    hazard,
    verbose,
    legend,
    legend_pos
) {
    # Keeping only culture of interest and reduce list into data.frame
    plot_data = lapply(plot_data, "[", culture)
    plot_data = stats::setNames(do.call(cbind, plot_data), names(plot_data))

    p_width = 16
    p_height = 8

    if (!is.null(output_name)) {
        name = file.path(
            path[["output"]][["figure"]],
            sprintf("%s_%s", culture, output_name)
        )
        switch(
            EXPR = tolower(tools::file_ext(name)), 
            pdf = grDevices::cairo_pdf(
                filename = name,
                width = p_width,
                height = p_height
            ),
            png = grDevices::png(
                filename = name, 
                width = p_width*2.54,
                height = p_height*2.54,
                units = "cm",
                res = 144
            ),
            jpg = grDevices::jpeg(
                filename = name, 
                width = p_width*2.54,
                height = p_height*2.54,
                units = "cm", 
                res = 144
            ),
            svg = grDevices::svg(
                filename = name,
                width = p_width,
                height = p_height
            ),
            stop(sprintf(
                "%s not recognized", 
                tolower(tools::file_ext(name))
            ))
        )
        on.exit(
            message("Output created at: ", name),
            add = TRUE,
            after = TRUE
        )
        on.exit(grDevices::dev.off(), add = TRUE, after = TRUE)
    }

    graphics::par(mar = c(7, 7, 5, 7))
    max_value = ceiling(max(rowSums(plot_data)))

    plot(
        x = 1,                 
        xlab = NA, 
        ylab = NA,
        xlim = c(0, 52), 
        ylim = c(
            0, 
            max_value*1.1
        ),
        type = "n",
        xaxt = "n",
        yaxt = "n"
    )
    week_axis(factor = 0.08, language = language)

    graphics::axis(
        2,
        at = seq(0, max_value, round_up_nice(max_value) / 10),
        labels = seq(0, max_value, round_up_nice(max_value) / 10),
        tck = 1,
        lty = 2,
        col = "grey",
        las = 2
    )

    ylab = {
        if (language == "EN") {"Damage amount (in \U20AC/ha)"}
        if (language == "FR") {"Montant de dommage (en \U20AC/ha)"}
    }
    graphics::mtext(
        ylab,
        side = 2,
        line = 5,
        cex = 1.2
    )

    for(i in colnames(plot_data)) {
        # Fill area between 0 and first plot values
        if(grep(i, colnames(plot_data)) == 1) {
            damage = c(plot_data[[i]][nrow(plot_data)],plot_data[[i]])
            graphics::polygon(
                c(0:52, rev(0:52)), c(damage, rep(0,53)),
                col = col[[i]],
                lty = 0
            )
        } else {
            damage_next = c(plot_data[[i]][nrow(plot_data)],plot_data[[i]])
            damage_next = mapply(
                "+",
                damage,
                damage_next,
                SIMPLIFY = FALSE
            )
            graphics::polygon(
                c(0:52, rev(0:52)), c(damage_next, rev(damage)),
                col = col[[i]],
                lty = 0
            )
            damage = damage_next
        }
    }

    if (isTRUE(legend)) {
        if (legend_pos %in% c("export_horizontal", "export_vertical")) {
            "Exporting legend will be available soon"
        } else {
            legend(
                x = legend_pos,
                legend = names(col),
                fill = col,
                bty = "o",
                bg = scales::alpha("white",0.5)
            )
        }
    }

    if (isFALSE(title_culture)) {
        graphics::title(
            main = sprintf("Culture : %s", culture),
            cex = 2
        )
    } else {
        graphics::title(
            main = sprintf("Culture : %s", title_culture[match(
                culture,
                title_culture[["code"]]
            ), "titre"]),
            cex = 2
        )
    }

    if (isTRUE(add_hazard_info)) { 
        graphics::mtext(
            sprintf(
                "duration = %s ; height = %s ; current = %s",
                duration,
                height,
                current
            ),
            side = 3,
            line = 0.2,
            cex = 1
        )
    }
}