#' @title Prepare actions and yields 
#' 
#' @description
#' `prepare_action_value()` prepares values of actions and yields for each crop
#' in `typo`.
#'
#' @param path character, path where to find action values.
#' @param typo data.frame of crop typology.
#' 
#' @details `prepare_action_value()` uses the order of elements in 
#' `path[["extent"]]` to extract action values and yields for each crop in 
#' coherence with `typo`.
#' 
#' If `typo` is missing, it will be read from the "floodam.csv" file in the
#' first extent of `path[["extent"]]`.
#' 
#' @return csv files of cost of actions, sale price and yield for each crop in
#' `typo`
#'
#' @export
#'
#' @encoding UTF-8

prepare_action_value = function(
    path,
    typo
) {
    # Reading typology from lowest level in path[["extent"]]
    if (missing(typo)) {
        typo = load_typo(path)
    }

    output = path[["output"]][["culture"]]

    # Reading available action_value paths
    action_value_all = find_data_path(
            path,
            name = "action.value.table.csv",
            field = "stat",
            all = TRUE
        )

    action_value_all = lapply(
        action_value_all,
        utils::read.csv2,
        stringsAsFactors = FALSE,
        row.names = 1
    )
    
    action_value = extract_action_value(action_value_all, typo)
    
    # Dealing with yield & weight
    ## Read available stat_<> files
    
    yield_all = find_file_path(
        path,
        name = "stat.",
        field = "stat",
        all = TRUE
    )

    # Testing that there is only one stat.<>.csv file in each extent
    test = lapply(
        path[["extent"]],
        function(x, string) {
            length(grep(x, string))
        },
        string = names(yield_all)
    ) > 1

    if (any(test)) {
        msg = paste(
            "There are more than one stat.<stat_name>.csv file in extent(s):",
            path[["extent"]][test]
        )
        stop(msg)
    }

    ## Read available weight_<> files 

    weight_all = stats::setNames(mapply(
        name = sprintf("weight.%s.floodam.csv", corename(yield_all)),
        extent = names(yield_all),
        find_weight_files,
        MoreArgs = list(
            path = path
        ),
        SIMPLIFY = FALSE
    ), names(yield_all))

    selection = unlist(lapply(weight_all, identical, y = character(0)))
    if (any(selection)) {weight_all[selection] = NULL}
    
    weight_all = lapply(unlist(weight_all), load_typology)

    yield_all = lapply(
        yield_all,
        utils::read.csv2,
        stringsAsFactors = FALSE,
        row.names = 1
    )

    yield = extract_yield(yield_all, weight_all, typo)

    # Combining and saving
    result = mapply(
        rbind,
        action_value[order(names(action_value))],
        yield[order(names(yield))],
        SIMPLIFY = FALSE
    )

    # Adapt attribution of extent depending on which folder the data was found
    selection = names(result) %in% typo[["type2"]]
    if (isFALSE(all(selection))) {result = result[selection]}
    save_list_df(result, path = output, pattern = "action.value")
}

extract_action_value = function(action_value, typo) {
    result = list()
    for (names in names(action_value)) {
        selection = row.names(action_value[[names]]) %in% unlist(typo)
        temp = make_list_of_df(action_value[[names]][selection,])
        temp = lapply(
            temp,
            function(x, names) {
                x[["extent"]] = names
                x[["source"]] = unique(x[["culture"]])
                return(x)
            },
            names
        )
        selection = !names(temp) %in% names(result)
        result = c(result, temp[selection])
    }
    # Copying missing action.value coherently with correspondence in typo
    ## Crops with action_value in extent

    todo = typo[!(typo[["type2"]] %in% names(result)),]
    selection = todo[todo[["type1"]] %in% names(result),]
    if (nrow(selection) != nrow(todo)) {
        missing = todo[!todo[["type1"]] %in% selection[["type1"]],"type1"]
        stop("Missing action.value for: ", paste(missing, collapse = ", "))
    }

    # Attribute the correct name coherently with correspondence in typo
    for (culture in todo[["type2"]]) {
        result[culture] = result[todo[
            todo[["type2"]] == culture, "type1"
        ]]
        result[[culture]][["culture"]] = culture
    }
    return(result)
}

extract_yield = function(yield_all, weight_all, typo, trim = 0) {
    yield = list()

    for (names in names(yield_all)) {
        result = apply(
            yield_all[[names]][grep("yield", colnames(yield_all[[names]]))],
            1,
            mean,
            na.rm = TRUE,
            trim = trim / ncol(yield_all[[names]])
        )
        result[is.nan(result)] = 0
        if (names %in% names(weight_all)) {
            weight = weight_all[[names]]
            result = data.frame(
                culture = colnames(weight),
                name = "yield",
                value = colSums(weight * result[rownames(weight)]),
                extent = names,
                stringsAsFactors = FALSE
            )
            selection = row.names(result) %in% unlist(typo)
            result = result[selection,]
            # Removing rows where yield = 0
            result = result[result[["value"]] > 0,]
            result = stats::setNames(
                split(result,seq(nrow(result))),
                row.names(result)
            )
        } else {
            if (isFALSE(any(names(result) %in% unlist(typo)))) next
            result = data.frame(
                culture = names(result),
                name = "yield",
                value = result,
                extent = names,
                stringsAsFactors = FALSE
            )
            selection = row.names(result) %in% unlist(typo)
            result = result[selection,]
            result = stats::setNames(
                split(result, seq(nrow(result))),
                row.names(result)
            )
        }
        selection = !names(result) %in% names(yield)
        yield = c(yield, result[selection])
    }
    yield = lapply(
        yield, 
        function(x) {
            x[["source"]] = unique(x[["culture"]])
            return(x)
        }
    )

    # Copying missing yield coherently with correspondence in typo
    todo = typo[!(typo[["type2"]] %in% names(yield)),]
    test = ! todo[["type1"]] %in% names(yield)

    if (any(test)) {
        missing = todo[test, "type1"]
        stop("Missing yield for: ", paste(missing, collapse = ", "))
    }

    # Attribute the correct name coherently with correspondence in typo
    for (culture in todo[["type2"]]) {
        yield[culture] = yield[todo[todo[["type2"]] == culture, "type1"]]
        yield[[culture]][["culture"]] = culture
    }
    return(yield)
}
