#' @title Prepare calendar
#' 
#' @description
#' `prepare_calendar()` extracts calendar information from "stat" folder to the 
#' `output`folder accordingly to the crops in `typo`.
#'
#' @param path list describing where to find useful paths. Result of
#' `init_path()`.
#' @param typo data.frame of crop typology. Should be found in `floodam.csv`.
#' 
#' @details `prepare_calendar()` uses the order of elements in `path[["extent"]]`
#' to load `typo` and to extract calendars for each crop.
#' 
#' If `typo` is missing, it will be read from the "floodam.csv" file in the
#' first extent of `path[["extent"]]`.
#' 
#' @return csv2 file of calendar for each crop present in `typo`.
#'
#' @export
#'
#' @encoding UTF-8

prepare_calendar = function(
    path,
    typo
) {
    # Reading typology from lowest level in path[["extent"]]
    if (missing(typo)) {
        typo = load_typo(path)
    }
    
    output = path[["output"]][["culture"]]

    # Declaring available calendar paths
    calendar_all = find_data_path(
        path,
        "calendar.csv",
        field = "stat",
        all = TRUE
    )
    
    # Reading calendar in coherence with path[["extent"]]
    calendar_all = lapply(
        calendar_all[names(calendar_all) %in% path[["extent"]]],
        utils::read.csv2,
        stringsAsFactors = FALSE
    )
    
    calendar = extract_calendar(calendar_all)

    # Saving calendars available at the type2 level
    selection = intersect(unlist(typo[["type2"]]), names(calendar))
    save_list_df(calendar[selection], path = output, pattern = "calendar")
    
    # Copying missing calendar coherently with correspondence in typo
    todo = typo[!(typo[["type2"]] %in% names(calendar)), ]
    selection = todo[todo[["type1"]] %in% names(calendar), ]

    if (nrow(selection) != nrow(todo)) {
        missing = todo[!todo[["type1"]] %in% selection[["type1"]], "type1"]
        stop("Missing calendar for: ", paste(missing, collapse = ", "))
    }
    
    invisible(mapply(
        utils::write.csv2,
        calendar[selection[["type1"]]],
        file.path(output, selection[["type2"]], "calendar.csv"),
        MoreArgs = list(
            row.names = FALSE
        )
    ))
}

extract_calendar = function(calendar_all) {
    calendar = list()
    for (cal in calendar_all) {
        result = split(cal, cal[["culture"]])
        selection = !names(result) %in% names(calendar)
        calendar = c(calendar, result[selection])
    }
    return(calendar)
}
