#' @title Prepare culture
#' 
#' @description
#' `prepare_culture()` copies vulnerability files from "damagement"
#' folder to the `output` folder accordingly to the crops in `typo`.
#'
#' @param path list describing where to find useful paths. Result of
#' `init_path()`.
#' @param typo data.frame of crop typology.
#' 
#' @details `prepare_culture()` uses the order of elements in `path[["extent"]]`
#' to find damagement files in coherence with `typo`.
#' 
#' If `typo` is missing, it will be read from the "floodam.csv" file in the
#' first extent of `path[["extent"]]`.
#' 
#' @return copy of vulnerability files in `path[["output"]][["culture"]]`.
#'
#' @export
#'
#' @encoding UTF-8

prepare_culture = function(
    path,
    typo
) {
    # Reading typology from lowest level in path[["extent"]]
    if (missing(typo)) {
        typo = load_typo(path)
    }
    
    input = stats::setNames(
        sapply(
            typo[["type1"]],
            find_data_path,
            path = path,
            field = "damagement"
        ),
        typo[["type2"]]
    )

    # Error if missing damagement files
    if (any(sapply(input, is.null))) {
        missing = names(input)[sapply(input, is.null)]
        stop("Missing damagement files for: ", paste(missing, collapse = ", "))
    }

    # Creating outputs and copying vulnerability files
    for (culture in names(input)){
        output = file.path(path[["output"]][["culture"]], culture)
        dir.create(output, showWarnings = FALSE)
        files = dir(input[[culture]])
        file.copy(file.path(input[[culture]], files), output, recursive = TRUE)
    }

    invisible()
}
