#' @title Prepare typology
#' 
#' @description
#' `prepare_typology()` creates weight files for each extent in 
#' `path[["extent"]]` according to the "guideline.csv" and 
#' "correspondance.<>.<>.csv" of these same extents.
#'
#' @param path list describing where to find useful paths.
#' @param classification character, name of classification to be used to assure
#' coherence between the wished classification and the correspondance files of
#' each extent. Default to NULL.
#' 
#' @details
#' If no guideline is found for an `extent` in `path[["extent"]]` nothing is
#' done.
#' 
#' @return .csv files of weights to be used in `calculate_damaging()`,
#' `combine_damaging()` and `prepare_action_value()`
#'
#' @export
#'
#' @encoding UTF-8

prepare_typology = function(path, classification = NULL) {
    # Verifying that there are typology folders
    test = all(unlist(lapply(
        path[["extent"]],
        function(x) {is.null(path[[x]][["typology"]])}
    )))
    if (test) {
        msg = paste(
            "No typology folder found in declared extents.",
            "Nothing is done."
        )
        stop(msg)
    }

    guideline_all = find_data_path(
        path,
        name = "guideline.csv",
        field = "typology",
        all = TRUE
    )
    
    if (is.null(guideline_all)) {
        msg = paste(
            "Could not find any guidelines to follow.",
            "Looking for 'guideline.csv' in 'typology' folder(s) in `path`."
        )
        stop(msg)
    }

    guideline_all = lapply(
        guideline_all,
        utils::read.csv2,
        colClasses = "character"
    )

    if (!is.null(classification)) {
        # Verifying classification directory exists
        test = is.null(path[["classification"]])
        msg = paste("No classification folder declared. See 'help(init_path)'.")
        if (test) stop(msg)

        # Verifying classification file exists
        test = !file.exists(file.path(
            path[["classification"]],
            paste0(classification, ".csv")
        ))
        msg = paste(
            "Missing classification file:",
            file.path(path[["classification"]], paste0(classification, ".csv"))
        )
        if (test) stop(msg)
    }
    
    for (extent in path[["extent"]]) {
        if (is.null(guideline_all[[extent]])) next
        apply(
            guideline_all[[extent]],
            1,
            treat_guideline,
            path[[extent]],
            path[["output"]][["typology"]][[extent]],
            classification = classification,
            extent = extent,
            path = path
        )
    }
}

#' @title Create weight file from guideline
#' 
#' @description `treat_guideline()` makes operations on `guideline` in order to
#' create a weight file
#' It calls `load_typology()` and `make_weight()`.
#' 
#' @param guideline data.frame of 1 observation of 4 variables.
#' @param input list of paths to folders where to find correspondance files and 
#' statistical data to be used to make weights.
#' @param output character giving the path to output directory.
#' @param trim integer, argument to be passed in `make_weight()`, default to 0.
#' @param classification character giving the name of classification to read in 
#' order to keep treated guidelines coherent.
#' @param extent character giving the name of the extent to be treated.
#' @param path list of useful paths.
#' 
#' @encoding UTF-8
#' 

treat_guideline = function(
    guideline,
    input,
    output,
    trim = 0,
    classification = classification,
    extent = extent,
    path = path
) {
    guideline = as.character(
        unlist(guideline[c("input", "output", "quantity", "intermediate")])
    )

    if (length(guideline) != 4) {
        stop(sprintf("Trouble with guidelines! Check %s", input))
    }
    
    if (!guideline[2] %in% c("floodam", classification)) {
        msg = paste0(
            sprintf(
                "Output values in '%s' ",
                file.path(path[[extent]][["typology"]], "guideline.csv")
            ),
            "should match one of c('floodam'",
            if (!is.null(classification)) {
                sprintf(", '%s')", classification)
            } else ")."
        )
        stop(sprintf("Trouble with guidelines! \n%s", msg))
    }

    correspondance_path = file.path(
        input[["typology"]],
        sprintf("correspondance.%s.%s.csv", guideline[1], guideline[2])
    )
    if (isFALSE(file.exists(correspondance_path)) && guideline[4] == "") {
        stop(sprintf(
            "Can not find correspondance! Check %s",
            correspondance_path
        ))
    }

    quantity = file.path(
        input[["stat"]],
        sprintf("%s.csv", if (guideline[4] == "") {
            guideline[1]
            } else {
            guideline[4]
        })
    )

    if (isFALSE(file.exists(quantity))) {
        stop(sprintf("Missing file ! Check %s", quantity))
    }

    path_weight = file.path(
        output,
        sprintf("weight.%s.%s.csv", guideline[1], guideline[2])
    )

    if (guideline[4] != "") {
        intermediate = file.path(
            input[["typology"]],
            sprintf("correspondance.%s.%s.csv", guideline[4], guideline[1])
        )
        temp = file.path(
            input[["typology"]],
            sprintf("correspondance.%s.%s.csv", guideline[4], guideline[2])
        )
        temp = load_typology(temp)
        intermediate = load_typology(intermediate)
        sel = rownames(temp)[rownames(temp) %in% rownames(intermediate)]
        temp = temp[sel, , drop = FALSE]
        intermediate = intermediate[sel, , drop = FALSE]
        correspondance = matrix(
            0,
            nrow = ncol(intermediate),
            ncol = ncol(temp),
            dimnames = list(colnames(intermediate), colnames(temp))
        )
        for (i in colnames(correspondance)) {
            rr = rownames(temp)[as.logical(temp[ , i])]
            cc = colnames(intermediate)[apply(
                intermediate[rr, , drop = FALSE],
                2,
                function(x) {any(as.logical(x), na.rm = TRUE)}
            )]
            correspondance[cc , i] = 1
        }
    } else {
        correspondance = load_typology(correspondance_path)
    }

    # Checking coherence between correspondance columns and classification
    
    if (identical(guideline[2], classification)) {
        class_path = file.path(
            path[["classification"]],
            sprintf("%s.csv", classification)
        )
        to_match = unlist(utils::read.csv2(class_path))

        selection = !colnames(correspondance) %in% to_match
        if (any(selection)) {
            msg = paste(
                sprintf("Check correspondance file:\n%s\n", correspondance_path),
                paste(colnames(correspondance)[selection], collapse = ", "),
                sprintf("not in '%s'", class_path)
            )
            stop(msg)
        }
    }

    quantity = as.matrix(utils::read.csv2(quantity, row.names = 1))
    selection = grep(guideline[3], colnames(quantity))
    quantity = quantity[, selection, drop = FALSE]

    if (guideline[4] != "") {
        intermediate = make_weight(
            intermediate,
            quantity,
            weight = FALSE,
            trim = trim
        )
        quantity = as.matrix(rowSums(intermediate, na.rm = TRUE))
    }

    utils::write.csv2(
        make_weight(correspondance, quantity),
        path_weight
    )
}