## In this file are grouped function that are used but should not be exported

### FUNCTIONS USED TO BUILD DAMAGING CURVE

#' @title Make array height and duration
#' 
#' @description
#' `array_hd()` makes the approximation on levels of height (h) and duration (d)
#' Called in `make_curve()`.
#' 
#' @param h vector of heights
#' @param d vector of durations
#' @param parameter data.frame of parameters of relation between damaging,
#' height, and duration
#' 
#' @return 2D-array (height, duration)
#'
#' @encoding UTF-8

array_hd = function(h, d, parameter) {
    fun = function(h, i) {
        x = calc_value(h, make_matrix(parameter[i, a$y]))
        y = calc_value(h, make_matrix(parameter[i, a$z]))
        stats::approx(x, y, d, rule = 2)$y
    }

    a = analyse_parameter(names(parameter))
    i = findInterval(as.numeric(h), parameter[, "x"])
    result = mapply(fun, as.numeric(h), i)
    dimnames(result) = list(duration = d, height = h)
    return(result)
}

#' @title Analyse parameter
#' 
#' @description
#' `analyse_parameter()` analyses parameters for `array_hd()`.
#' 
#' @param name TO BE PRECISED
#' 
#' @return list, TO BE PRECISED
#'
#' @encoding UTF-8
#' 

analyse_parameter = function(name) {
    n = length(grep("ys", name))
    if (length(unique(c(n, length(grep("yc", name)),
        length(grep("zs", name)), length(grep("zc", name))))) != 1)
        stop("Check colnames !")

    list(
        y = paste(rep(c("ys", "yc"), n), rep(seq(n), each = 2), sep = ""),
        z = paste(rep(c("zs", "zc"), n), rep(seq(n), each = 2), sep = "")
    )
}

#' @title Combine parameters
#' 
#' @description
#' `calc_value()` calculates how parameters should be combined in `array_hd()`.
#' 
#' @param x TO BE PRECISED
#' @param parameter TO BE PRECISED
#' 
#' @return TO BE PRECISED
#'
#' @encoding UTF-8
#' 

calc_value = function(x, parameter) {parameter[, 1] * x + parameter[, 2]}

#' @title Make matrix
#' 
#' @description
#' `make_matrix()` constructs matrix for `array_hd()`.
#' 
#' @param x TO BE PRECISED
#' 
#' @return TO BE PRECISED
#'
#' @encoding UTF-8
#' 

make_matrix = function(x) {matrix(x[!is.na(x)], ncol = 2, byrow = TRUE)}

### FUNCTIONS USED TO FIT DATA FORMAT

#' @title Fit calendar
#' 
#' @description
#' `fit_calendar()` fits calendar format, TO BE PRECISED, called in 
#' `calculate_damaging()`
#' 
#' @param x TO BE PRECISED
#' 
#' @return TO BE PRECISED
#'
#' @encoding UTF-8
#' 

fit_calendar = function(x) {
    # is start.week well ordered?
    sw = x$start.week
    if (any(sw != sort(sw))) {
        i = which(sw == min(sw))
        sw = sw[c(seq(i, length(sw)), seq(1, i - 1))]
        if (any(sw != sort(sw))) stop("start.week not well ordered!")
    }
    # Adding end.week
    x$end.week = calendarize(c(x$start.week[-1], x$start.week[1]) - 1)

    # Adding row names corresponding to phase (unique)
    rownames(x) = as.character(x$phase)

    return(x[c("start.week", "end.week")])
}

#' @title Find curve
#' 
#' @description
#' `find_curve()` TO BE PRECISED, called in `array_week()`
#' 
#' @param x TO BE PRECISED
#' @param calendar TO BE PRECISED
#' @param name TO BE PRECISED, default to TRUE
#' 
#' @return TO BE PRECISED
#'
#' @encoding UTF-8
#' 

find_curve = function(x, calendar, name = TRUE) {
    result = mapply(
        in_period,
        as.numeric(x),
        calendar$start.week,
        calendar$end.week
    )
    if (isTRUE(name)) {
        result = rownames(calendar)[result]
    } else {
        result = x[result]
    }

    return(result)
}

### FUNCTIONS USED TO CALCULATE DAMAGE DEPENDING ON CONDITIONS

#' @title Apply test
#' 
#' @description
#' `apply_test()` TO BE PRECISED, called in `calculate_damaging()`
#' 
#' @param test TO BE PRECISED
#' @param damaging TO BE PRECISED
#' @param calendar TO BE PRECISED
#' @param default TO BE PRECISED, default to NA
#' 
#' @return TO BE PRECISED
#'
#' @encoding UTF-8
#' 

apply_test = function(test, damaging, calendar, default = NA) {
    size = nrow(damaging)
    result = data.frame(row.names = seq(size))
    result[unique(as.character(test$object))] = default

    test$start.week = calendar[as.character(test$start.phase), "start.week"]
    test$end.week = calendar[as.character(test$end.phase), "end.week"]

    discount = "start.year" %in% names(test) & "end.year" %in% names(test)
    test$discount = if (isTRUE(discount))
        mapply(cumsum_discount_weight, test$start.year, test$end.year) else 1

    for (i in seq(nrow(test))) {
        o = as.character(test[i, "object"])
        v = test[i, "value"]
        f = as.character(test[i, "factor"])
        var.test = as.character(test[i, "var.test"])
        var.min = test[i, "var.min"]
        var.max = test[i, "var.max"]
        start.week = test[i, "start.week"]
        end.week = test[i, "end.week"]
        discount = test[i, "discount"]

        selection = rep(TRUE, size)
        if (var.test %in% names(damaging))
            selection = selection &
                damaging[[var.test]] >= var.min & 
                damaging[[var.test]] <= var.max
        if (!is.na(start.week))
            selection = selection &
                in_period(damaging[["week"]], start.week, end.week)

        if (f %in% names(damaging))
            result[[o]][selection] = v * damaging[[f]][selection] * discount
        else
            result[[o]][selection] = v * as.numeric(f) * discount
    }
    return(result)
}

### OTHER FUNCTIONS

#' @title Sequence period
#' 
#' @description
#' `seq_period()` makes a sequence for a period from a starting date, end date 
#' and `size` which corresponds to the maximum value `s` or `e` can take.
#' Called in `load_hazard()`.
#' 
#' @param s numeric, starting date of period.
#' @param e numeric, end date of period.
#' @param size integer, maximum value of period, default to 52.
#' 
#' @return vector of integer corresponding to dates in period
#'
#' @encoding UTF-8

seq_period = function(s, e, size = 52) {if (s <= e) s:e else c(s:52, 1:e)}


#' @title Calendarize
#' 
#' @description
#' `calendarize()` TO BE PRECISED. Called in `fit_calendar()`.
#' 
#' @param x TO BE PRECISED
#' @param period integer, maximum value of period, default to 52.
#' 
#' @return TO BE PRECISED
#'
#' @encoding UTF-8
#'

calendarize = function(x, period = 52) {
    x = x %% period
    x[x == 0] = period
    return(x)
}

#' @title Remove extension from name
#' 
#' @description
#' `corename()` removes extension from name. Called in `aggregate_damage()` and
#' `load_parameter()`.
#' 
#' @param x character, name to be corrected
#' 
#' @return corrected name without the extension.
#'
#' @examples
#' 
#' x = "my_file.txt"
#' x = floodam.agri:::corename(x)
#' print(x)
#' 
#' @encoding UTF-8
#'

corename = function(x) {
    gsub("[.][[:alnum:]]+$", "", basename(x))
}

#' @title Extract damage
#' 
#' @description
#' `extract_damage()` extracts given type of damage from a data.frame of damage
#' 
#' @param x data.frame
#' @param d character, column name, default to "damage.total"
#' 
#' @return vector of damage
#'
#' @encoding UTF-8

extract_damage = function(x, d = "damage.total") {
    x[[d]]
}

#' @title Transform list of array to data.frame
#' 
#' @description
#' `listofarray_to_df()` transforms list of array to data.frame.
#' Called in `make_curve()`
#' 
#' @param x list of arrays to be transformed into data.frame
#' 
#' @return data.frame
#'
#' @encoding UTF-8
#'

listofarray_to_df = function(x) {
    result = list()

    xx = x[[1]]
    n = dimnames(xx)
    result[[names(n)[1]]] = rep(n[[1]], times = prod(dim(xx)[-seq(1)]))
    result[[names(n)[2]]] = rep(
        n[[2]],
        times = prod(dim(xx)[-seq(2)]),
        each = prod(dim(xx)[seq(1)])
    )
    result[[names(n)[3]]] = rep(
        n[[3]],
        times = prod(dim(xx)[-seq(3)]),
        each = prod(dim(xx)[seq(2)])
    )
    result[[names(n)[4]]] = rep(n[[4]], each = prod(dim(xx)[seq(3)]))
    result = lapply(result, as.numeric)

    for (i in names(x)) result[[i]] = as.vector(x[[i]])
    return(data.frame(result))
}

#' @title Make name to correct format
#' 
#' @description
#' `make_name()` changes format of name, called in 
#' `load_parameter()`
#' 
#' @param x character of name to be changed
#' @param label character of name to be changed
#' 
#' @return character of parameter name with correct format.
#'
#' @encoding UTF-8
#'

make_name = function(x, label) {
    r = gsub(label, "", x)
    r = gsub("_", ".", r)
    return(r)
}

#' @title TO BE PRECISED
#' 
#' @description
#' `inv()` tests if i in names of x. If Test is true calculate return 1 - x[[i]]
#' Else return NULL (and so can be used not to add a column to a data.frame
#' Called in `calculate_damaging()`.
#' 
#' @param x TO BE PRECISED
#' @param i TO BE PRECISED
#' 
#' @return TO BE PRECISED
#'
#' @encoding UTF-8
#'

inv = function(x, i) {
    result = if (i %in% names(x)) result = 1 - x[[i]] else NULL
    return(result)
}

#' @title Make ratio week period
#' 
#' @description
#' `ratio_week_period()` TO BE PRECISED, called in `calculate_damaging()`.
#' 
#' @param week TO BE PRECISED
#' @param calendar TO BE PRECISED
#' @param period character, name of the period, default to "harvest"
#' 
#' @return TO BE PRECISED
#'
#' @encoding UTF-8
#'

ratio_week_period = function(week, calendar, period = "harvest") {
    result = rep(NA, length(week))
    if (period %in% rownames(calendar))
    {
        selection = in_period(
            week,
            calendar[period, "start.week"],
            calendar[period, "end.week"]
        )
        if (any(selection)) {
            result[selection] = mapply(
                length_period,
                week[selection],
                calendar[period, "end.week"]) / length_period(
                    calendar[period, "start.week"],
                    calendar[period, "end.week"]
                )
        }
    }
    return(result)
}

### Function for treatment on parameters

#' @title Make ratio week period
#' 
#' @description
#' `action_value()` replaces definition of value by their evaluation using
#' link (TO BE PRECISED), called in `load_parameter()`.
#' 
#' @param definition TO BE PRECISED
#' @param link TO BE PRECISED
#' 
#' @return TO BE PRECISED
#'
#' @encoding UTF-8
#'

action_value = function(definition, link) {
    definition = as.character(definition)
    link$name = as.character(link$name)
    for (i in seq(nrow(link)))
        assign(link$name[i], link$value[i])
    sapply(definition, function(x){eval(parse(text = x))})
}

#' @title Check phase
#' 
#' @description
#' `check_phase()` checks phasein calendar for specified parameter (TO BE 
#' PRECISED)
#' 
#' @param parameter TO BE PRECISED
#' 
#' @return list, TO BE PRECISED
#'
#' @encoding UTF-8
#'

check_phase = function(parameter) {
    result = parameter$calendar["phase"]
    result$start.phase =
        result$phase %in% parameter$action$start.phase |
        result$phase %in% parameter$damaging.complement$start.phase |
        result$phase %in% parameter$rule$start.phase
    result$end.phase =
        result$phase %in% parameter$action$end.phase |
        result$phase %in% parameter$damaging.complement$end.phase |
        result$phase %in% parameter$rule$end.phase
    result
}

#' @title Source directory
#' 
#' @description
#' `source_dir()` TO BE PRECISED
#' 
#' @param x character indicating path where to source directory, TO BE PRECISED
#' @param ... arguments to be passed to `base::source()``
#' 
#' @return vector of damage
#'
#' @encoding UTF-8
#' 

source_dir = function(x, ...) {
    invisible(lapply(file.path(x, dir(x)), source, localv = .GlobalEnv, ...))
}

#' @title Load floodam.csv
#' 
#' @description
#' `load_typo()` loads floodam.csv at the lowest possible level of 
#' `path[["extent"]]`. floodam.csv should be stored in a subdirectory named 
#' "typology" of one of `path[["extent"]]`.
#' 
#' @param path list of useful paths.
#' 
#' @return data.frame of type1 and type2 crops to use in simulation.
#'
#' @encoding UTF-8
#' 

load_typo = function(path) {
    typo = find_data_path(path, name = "floodam.csv", field = "typology")
    if (is.null(typo)) {
        stop("typo is missing and no 'floodam.csv' file could be found.")
    }
    typo = utils::read.csv2(typo)
    return(typo)
}

#' @title plot week axis
#' 
#' @description
#' `week_axis()` adds labels to x axis on plots
#' 
#' @param y_height numeric, value of plot y height. Default to `graphics::par("usr")[4]`.
#' @param y_min numeric, value of minimum y height. Default to `graphics::par("usr")[3]`.
#' @param weeks numeric, number of weeks to plot.
#' @param factor numeric, factor used to determine position of calendar text.
#' @param language character, one of c("FR", "EN")
#' 
#' @return weekly labels to x axis on plot.
#'
#' @encoding UTF-8
#' 

week_axis = function(
    y_height = graphics::par("usr")[4],
    y_min = graphics::par("usr")[3],
    weeks = 52,
    factor = 0.15,
    language = "EN"
) {
    if (language == "EN") {
        # Set back to locale when function exits
        locale = Sys.getlocale("LC_TIME")
        on.exit(Sys.setlocale("LC_TIME", locale))

        # set language for dates on x axis
        Sys.setlocale("LC_TIME", "en_US.UTF8")
    }

    if (language == "FR") {
        # Set back to locale when function exits
        locale = Sys.getlocale("LC_TIME")
        on.exit(Sys.setlocale("LC_TIME", locale))

        # set language for dates on x axis
        Sys.setlocale("LC_TIME", "fr_FR.UTF-8")
    }
        
    {
        # Pass weeks to dates
        weeks = dates = pos = seq(2, weeks, 2)
        selection = weeks > 52
        weeks[selection] = weeks[selection] - 52

        dates[!selection] = format(
            as.Date(paste(weeks[!selection], 2023, 3), '%U %Y %w'),
            "%b-%d"
        )
        dates[selection] = format(
            as.Date(paste(weeks[selection], 2024, 3), '%U %Y %w'),
            "%b-%d"
        )
    }
        graphics::axis(
            1,
            at = pos,
            labels = weeks,
            tck = 1,
            lty = 2,
            col = "grey",
            cex.axis = 1.2
        )
        
        graphics::text(
            x = pos,
            y = y_min-factor*diff(c(y_min, y_height)),
            xpd = TRUE,
            labels = dates,
            srt = 35,
            cex = 1.2,
            adj = 0.8
        )

        graphics::mtext(
            "Date",
            1,
            line = 5.5,
            cex = 2
        )
}

#' @title round values nicely
#' 
#' @description
#' `round_up_nice()` rounds a numeric value
#' 
#' @param x numeric, value to be rounded.
#' @param nice numeric vector with nice base values from 1 to 10
#' 
#' @return rounded value of x.
#'
#' @encoding UTF-8
#' 

round_up_nice = function(x, nice = c(1, 2, 4, 5, 6, 8, 10)) {
    if (length(x) != 1) stop("'x' must be of length 1")
    10^floor(log10(x)) * nice[[which(x <= 10^floor(log10(x)) * nice)[[1]]]]
}
