## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.align = "center"
)

## ----setup--------------------------------------------------------------------
#> Loading required package: floodam.agri
library(floodam.agri)

#> Setting up environment paths (inputs and outputs)
output = file.path(tempdir(), "get_started")
path = init_path(
  output = output,
  default = TRUE
)

## ----data-preparation---------------------------------------------------------
#> Extracting damagement files to output
prepare_culture(path)

#> Extracting and copying calendars to output
prepare_calendar(path)

#> Building weight files
prepare_typology(path)

#> Preparing necessary action values and yields
prepare_action_value(path)

## ----damage-function----------------------------------------------------------
#> Simplifying LEVEL
level = LEVEL
level[["duration"]] = seq(0, 5, 1)
level[["height"]] = seq(0, 100, 10)
level[["week"]] = seq(2, 52, 2)

#> Calculating damage function
damaging = calculate_damaging(
  path,
  culture = c("greenbean", "greenpea"),
  level = level,
  update.hazard = TRUE,
  retrieve = TRUE
)

## ----damaging-sample, echo = FALSE--------------------------------------------
DT::datatable(
  head(damaging[["greenbean"]], 5000),
  caption = "Result of `calculate_damaging()`: raw total disaggregated damage of a greenbean plot (first 5000 rows).",
  rownames = FALSE,
  options = list(
    pageLength = 10, autoWidth = TRUE,
    lengthMenu = c(5, 10, 15, 20, 25, 50, 100)
  )
)

## ----combine-damage-function--------------------------------------------------
#> Combine damage to new classification
combine = combine_damaging(
  path = path,
  damage.culture = damaging,
  retrieve = "total" # one of c("total", "yield.annual", "yield.differed",
  # "replanting", "soil", material")
)

## ----combine-sample, echo = FALSE---------------------------------------------
DT::datatable(
  head(combine, 5000),
  caption = "Result of `combine_damaging()`: raw total damage with RPG classification (first 5000 rows).",
  rownames = FALSE,
  options = list(
    pageLength = 10,
    autoWidth = TRUE,
    lengthMenu = c(5, 10, 15, 20, 25, 50, 100)
  )
)

## ----aggregate-damage---------------------------------------------------------
#> Aggregate damage
agg_damaging = aggregate_damage(
  path,
  damage_origin = "total" # default
)

## ----aggregate-sample, echo = FALSE-------------------------------------------
DT::datatable(
  agg_damaging,
  caption = "Result of `aggregate_damage()`.",
  rownames = FALSE,
  options = list(
    pageLength = 10, autoWidth = TRUE,
    lengthMenu = c(5, 10, 15, 20, 25, 50, 100)
  )
)

## ----visualisation, results = 'hide', fig.keep = 'all', fig.width = 7, fig.height = 7----
#> Show plot
plot_aggregated_damage(
  path = path,
  damage = "total.aggregate", # default
  culture = "greenpea",
  current = "light", # default
  duration = c("short", "long"), # default
  legend = "include"
)

## ----visualisation-optimal, results = 'hide', fig.keep = 'all', fig.width = 7, fig.height = 7----
HAZARD_AGGREGATE[["week"]] = data.frame(
  min = c(40, 14, 27),
  max = c(13, 26, 39),
  row.names = c("autumn-winter", "spring", "summer")
)
aggregate_damage(
  path,
  hazard_agg = HAZARD_AGGREGATE,
  damage_origin = "total" # default
)

#> Show plot
plot_aggregated_damage(
  path = path,
  damage = "total.aggregate", # default
  culture = "greenpea",
  current = "light", # default
  duration = c("short", "long"), # default
  legend = "include"
)

## ----calendar, results = 'hide', fig.keep = 'all', fig.width = 15, fig.height = 8----
#> Show plot
plot_crop_calendar(
  path = path,
  culture = c("greenbean", "greenpea")
)

