## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.align = "center"
)
path_data = system.file(
    "extdata",
    "vignettes",
    "winery",
    package = "floodam.agri"
)

## ----winery, out.width = '60%', echo = FALSE----------------------------------
knitr::include_graphics("img/winery_farm.png")

## ----echo = FALSE-------------------------------------------------------------
dataset = read.csv2(file.path(path_data, "typology", "floodam.csv"))

knitr::kable(dataset, caption = "floodam.csv", booktabs = TRUE) |>
  kableExtra::kable_styling(position = "center")

## ----echo = FALSE-------------------------------------------------------------
dataset = read.csv2(file.path(path_data, "typology", "title.culture.csv"))

knitr::kable(dataset, caption = "title.culture.csv", booktabs = TRUE) |>
  kableExtra::kable_styling(position = "center")

## ----echo = FALSE-------------------------------------------------------------
dataset = read.csv2(file.path(path_data, "stat", "stat.local.csv"))

knitr::kable(dataset, caption = "stat.local.csv", booktabs = TRUE) |>
  kableExtra::kable_styling(position = "center")

## ----echo = FALSE-------------------------------------------------------------
dataset = read.csv2(file.path(path_data, "stat", "calendar.csv"))

DT::datatable(
  dataset,
  caption = "calendar.csv",
  rownames = FALSE,
  options = list(
    pageLength = 10, autoWidth = TRUE,
    lengthMenu = c(5, 10, 15, 20, 25, 50, 100)
  )
)

## ----echo = FALSE-------------------------------------------------------------
dataset = read.csv2(file.path(path_data, "stat", "action.value.table.csv"))
selection = c(TRUE, !apply(dataset[-1], 2, function(x) all(is.na(x))))

knitr::kable(
  dataset[selection],
  caption = "Sample of action.value.table.csv (fields needed for vineyard crops)",
  booktabs = TRUE
) |>
  kableExtra::kable_styling(position = "center")

## ----setup--------------------------------------------------------------------
#> Loading required package: floodam.agri
library(floodam.agri)

#> Setting up environment paths (inputs and outputs)
output = file.path(tempdir(), "winery")
input = setNames(
    system.file("extdata", "vignettes", "winery", package = "floodam.agri"),
    "winery"
) # setting a name to input helps keep track of where data has been found
path = init_path(
  output = output,
  input = input,
  default = TRUE # default value, TRUE is obligatory to fetch the default
  # damagement files
)

## ----data-preparation---------------------------------------------------------
#> Extracting damagement files to output
prepare_culture(path)

#> Extracting and copying calendars to output
prepare_calendar(path)

#> Building weight files
prepare_typology(path)

#> Preparing necessary action values and yields
prepare_action_value(path)

## ----results = 'hide', fig.keep = 'all', fig.width = 15, fig.height = 8-------
plot_crop_calendar(
  path,
  title_culture = TRUE,
  reference = "vineyard_floodam", # the reference crop appears above other crops
  order_phase = "harvest" # order crops by harvest date
)

## -----------------------------------------------------------------------------
#> Simplifying LEVEL
level = LEVEL
level[["duration"]] = seq(0, 5, 1)
level[["height"]] = seq(0, 100, 10)

#> Calculating damage function
damaging = calculate_damaging(
  path,
  culture = NULL,
  level = level,
  update.hazard = TRUE,
  retrieve = TRUE
)

## -----------------------------------------------------------------------------
#> Combine damage into one file for each damage type
combine = combine_damaging(
  path = path,
  damage.culture = damaging,
  retrieve = "total" # one of c("total", "yield.annual", "yield.differed",
  # "replanting", "soil", material")
)

## ----fig.width = 16, fig.height = 8-------------------------------------------
#> Show plot
plot_damage_function(
  path = path,
  damage = "total",
  legend = TRUE,
  main = "Damage curves for all grape varieties of the winery",
  current = 1,
  duration = 4,
  height = 100
)

## ----fig.width = 16, fig.height = 10------------------------------------------
#> Show plot
plot_disaggregated_damage_function(
  path = path,
  culture = "variety_1",
  current = 1,
  duration = 4,
  height = 100
)

## ----aggregate-damage---------------------------------------------------------
#> Aggregate damage
agg_damaging = aggregate_damage(
  path,
  damage_origin = "total" # default
)

## ----fig.width = 7, fig.height = 7--------------------------------------------
#> Show plot
plot_aggregated_damage(
  path = path,
  damage = "total.aggregate", # default
  culture = "variety_1",
  current = "light", # default
  duration = c("short", "long"), # default
  legend = "include"
)

